package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/AttributeEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>AttributeAddEditForm</code>
 * object.
 * 
 * @author David Tobey
 * 
 */
public class AttributeEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(AttributeEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		AttributeAddEditForm baseForm = (AttributeAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		AttributeProcessor processor = (AttributeProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"attributeProcessorImplementer");
		Attribute attribute = processor.loadAttributeFromID(PropertyUtils
				.describe(form));
		BeanUtils.copyProperties(baseForm, attribute);

		// Load the parent if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		Product product = null;
		if (log.isDebugEnabled())
			log.debug("Checking for parent: " + baseForm.getProductID());
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor parentProcessor =  bof
					.createObject(ProductProcessor.class);
			product = parentProcessor.loadProductFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("product", product);
		}

		// Load in this attribute's options for the default option drop down
		Collection attributeOptions = attribute.getAttributeOptions();
		baseForm.getRequest()
				.setAttribute("attributeOptions", attributeOptions);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("attributeEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Attributes", "Attribute Details" });
			request.setAttribute("breadcrumbs", bcs);

			// Create child childSubNavigation collection
			Collection childsn = Arrays.asList(new String[] {
					"Attribute Details", "Options", "Add Option" });
			request.setAttribute("childSubNavigation", childsn);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Attributes",
					"Attribute Details" });
			request.setAttribute("breadcrumbs", bcs);
		}

		request.setAttribute("attribute", attribute);
		request.setAttribute("attributeAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
