package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/ManufacturerAdd.do". Processes a request to add a record to
 * the system.
 * 
 * @author David Tobey
 */
public class ManufacturerAddAction extends ManufacturerEditAction {

	static Log log = LogFactory.getLog(ManufacturerAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Add the record
		ManufacturerAddEditForm baseForm = (ManufacturerAddEditForm) form;
		ManufacturerProcessor processor = (ManufacturerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"manufacturerProcessorImplementer");
		Map results = processor.addManufacturer(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Reload manufacturers in the application scope.
			ProductInitializer pi = new ProductInitializer();
			pi.setServlet(servlet);
			pi.initializeManufacturers(baseForm.getBusinessObjectFactory());

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Manufacturer newRecord = (Manufacturer) results.get("newRecord");

				// Set success message
				baseForm.getMessages().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.editNewRecord", "ManufacturerEditForm.do?manufacturerID="
								+ newRecord.getManufacturerID()));

				// Optionally update the Lucene index
				updateAdvancedSearchIndex(baseForm, newRecord.getManufacturerID());
			}

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");
		} else if (results != null && results.get("existingRecord") != null) {

			// Set the error messages
			Manufacturer existingRecord = (Manufacturer) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = existingRecord.getCode();
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode"), duplicateField, existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord", "ManufacturerEditForm.do?productID="
							+ existingRecord.getManufacturerID(), duplicateField, existingCode));
		}
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		return mapping.findForward("success");

	}
}
