package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/ProductEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>ProductAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class ProductEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ProductAddEditForm baseForm = (ProductAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Load the parent if an id is present
		Category category = null;
		if (baseForm.getCategoryID() != null
				&& baseForm.getCategoryID().length() > 0) {
			CategoryProcessor categoryProcessor = (CategoryProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"categoryProcessorImplementer");
			category = categoryProcessor.loadCategoryFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("category", category);
		}

		// Get the record to edit
		ProductProcessor processor = (ProductProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"productProcessorImplementer");
		Product product = null;
		if (baseForm.getProductID() != null) {
			product = processor.loadProductFromID(PropertyUtils.describe(form));
		}
		if (product == null && baseForm.getCode() != null) {
			product = processor.loadProductFromCode(PropertyUtils.describe(form));
		}
		BeanUtils.copyProperties(baseForm, product);

		// Load all categories for the primary category drop down.
		CategoryProcessor catProcessor = (CategoryProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"categoryProcessorImplementer");
		Collection categories = catProcessor.loadAllCategories();
		baseForm.getRequest().setAttribute("categories", categories);

		// Load all manufacturers for the manufacturer drop down.
		ManufacturerProcessor manfProcessor = (ManufacturerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"manufacturerProcessorImplementer");
		Collection manufacturers = manfProcessor.loadAllManufacturers();
		request.setAttribute("manufacturerCollection", manufacturers);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("productEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Products",
				"Product Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("product", product);
		request.setAttribute("productAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
