package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * <code>/administrator/SKUAddForm.do</code>. Creates an empty
 * <code>SKUAddEditForm</code> object.
 * 
 * @author David Tobey
 */
public class SKUAddFormAction extends BaseAction {

	static Log log = LogFactory.getLog(SKUAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		SKUAddEditForm baseForm = (SKUAddEditForm) form;

		if (request.getAttribute("reset") != null
				&& request.getAttribute("reset").equals("true")) {
			Class classDefinition = baseForm.getClass();
			baseForm = (SKUAddEditForm) classDefinition.newInstance();
		}

		baseForm.initializeProperties(mapping, request);
		request.setAttribute("skuAddEditForm", baseForm);
		baseForm.setProductID(request.getParameter("productID"));
		baseForm.setParentIDField(request.getParameter("parentIDField"));

		// Load the parent if an id is present
		Product product = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor = (ProductProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"productProcessorImplementer");
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Load attributes for the attribute/option drop down.
		Collection attributes = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor = (ProductProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"productProcessorImplementer");
			attributes = productProcessor.getAllAttributesAndOptions(Integer
					.parseInt(baseForm.getProductID()));
		} else {
			AttributeProcessor attributeProcessor = (AttributeProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"attributeProcessorImplementer");
			attributes = attributeProcessor.loadAllAttributes();
		}
		request.setAttribute("attributes", attributes);

		// Load products for the product drop down.
		ProductProcessor productProcessor = (ProductProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"productProcessorImplementer");
		Collection productNamesCodesIDs = productProcessor
				.loadAllProductNamesCodesIDs();
		request.setAttribute("productNamesCodesIDs", productNamesCodesIDs);

		// Navigation
		if (request.getParameter("parentIDField") != null
				&& request.getParameter("parentIDField").equals("productID")) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "SKUs", "Add SKU" });
			request.setAttribute("breadcrumbs", bcs);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "SKUs",
					"Add SKU" });
			request.setAttribute("breadcrumbs", bcs);

		}

		return mapping.findForward("success");
	}
}
