package com.softslate.commerce.administrator.shipping;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.shipping.ShippingMethod;
import com.softslate.commerce.businessobjects.shipping.ShippingMethodProcessor;
import com.softslate.commerce.businessobjects.shipping.ShippingRate;
import com.softslate.commerce.businessobjects.shipping.ShippingRateProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/ShippingRateEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>ShippingRateAddEditForm</code>
 * object.
 * 
 * @author David Tobey
 */
public class ShippingRateEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(ShippingRateAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ShippingRateAddEditForm baseForm = (ShippingRateAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		ShippingRateProcessor processor = (ShippingRateProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"shippingRateProcessorImplementer");
		ShippingRate shippingRate = processor
				.loadShippingRateFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, shippingRate);

		// Load the parent if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		ShippingMethod shippingMethod = null;
		if (baseForm.getShippingMethodID() != null
				&& baseForm.getShippingMethodID().length() > 0) {
			ShippingMethodProcessor parentPprocessor =  bof
					.createObject(ShippingMethodProcessor.class);
			shippingMethod = parentPprocessor
					.loadShippingMethodFromID(PropertyUtils.describe(baseForm));
			request.setAttribute("shippingMethod", shippingMethod);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("shippingEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home",
				"Shipping Methods", "Shipping Method Details",
				"Shipping Rates", "Shipping Rate Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("shippingRate", shippingRate);
		request.setAttribute("shippingRateAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
