package com.softslate.commerce.businessobjects.customer;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.product.Product;

/**
 * Interface representing an product under a wish list in the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CustomerWishListProduct</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "customerWishListProductImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "customerWishListProductImplementer" is
 * {@link com.softslate.commerce.businessobjects.customer.CustomerWishListProductBean}.
 * 
 * @author Jason McWilliams
 * 
 * @see com.softslate.commerce.businessobjects.customer.CustomerWishListProduct
 */
@Audited
@AuditTable("AudCustomerWishListProduct")
public interface CustomerWishListProduct extends BusinessObject {
	abstract int getCustomerWishListProductID();

	abstract void setCustomerWishListProductID(int customerWishListProductID);

	abstract int getCustomerWishListID();

	abstract void setCustomerWishListID(int customerWishListID);

	abstract int getProductID();

	abstract void setProductID(int productID);

	abstract Product getProduct();

	abstract void setProduct(Product product);

	abstract CustomerWishList getCustomerWishList();

	abstract void setCustomerWishList(CustomerWishList customerWishList);

}
