package com.softslate.commerce.businessobjects.order;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.events.ChangingOrderDeliveryEvent;
import com.softslate.commerce.daos.order.OrderDeliveryDAO;
import com.softslate.commerce.daos.order.OrderDeliveryGatewayDAO;

/**
 * Processes business logic having to do with order deliveries. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting order deliveries.
 * <p>
 * <code>BasicOrderDeliveryProcessor</code> is the default implementation of the
 * <code>OrderDeliveryProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicOrderDeliveryProcessor extends BaseBusinessProcessor
		implements OrderDeliveryProcessor {

	static Log log = LogFactory.getLog(BasicOrderDeliveryProcessor.class);

	public Map loadOrderDeliveriesAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading order deliveries and count");
		OrderDeliveryGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(OrderDeliveryGatewayDAO.class);
		return gatewayDAO.loadOrderDeliveriesAndCount(parameters);
	}

	public Map addOrderDelivery(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("address1");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding order delivery: " + code);
		}
		Map results = new HashMap();
		OrderDelivery orderDelivery =  getBusinessObjectFactory()
				.createObject(OrderDelivery.class);
		BeanUtils.copyProperties(orderDelivery, parameters);
		if (orderDelivery.getOrder() == null) {
			Order order = (Order) getBusinessObjectFactory().createObject(
					"orderImplementer");
			order.setOrderID(orderDelivery.getOrderID());
			orderDelivery.setOrder(order);
		}
		OrderDeliveryDAO dao = (OrderDeliveryDAO) getDaoFactory().createDAO(
				"orderDeliveryDAOImplementer");
		dao.setOrderDelivery(orderDelivery);
		dao.insertOrderDelivery(true);
		results.put("newRecord", dao.getOrderDelivery());
		return results;
	}

	public OrderDelivery loadOrderDeliveryFromID(Map parameters) throws Exception {
		return loadOrderDeliveryFromID((Integer) ConvertUtils.convert(parameters.get("orderDeliveryID"), Integer.class));
	}

	public OrderDelivery loadOrderDeliveryFromID(int orderDeliveryID) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading order delivery from id");
		OrderDelivery orderDelivery =  getBusinessObjectFactory()
				.createObject(OrderDelivery.class);
		OrderDeliveryDAO dao = (OrderDeliveryDAO) getDaoFactory().createDAO(
				"orderDeliveryDAOImplementer");
		orderDelivery.setOrderDeliveryID(orderDeliveryID);
		dao.setOrderDelivery(orderDelivery);
		dao.loadOrderDeliveryFromID();
		return dao.getOrderDelivery();
	}

	public Map editOrderDelivery(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("address1");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing order delivery: " + code);
		}
		OrderDelivery orderDelivery = loadOrderDeliveryFromID(parameters);
		Map results = postChangingOrderDeliveryEvent(orderDelivery, parameters);
		BeanUtils.copyProperties(orderDelivery, parameters);
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		order.setOrderID(orderDelivery.getOrderID());
		orderDelivery.setOrder(order);
		OrderDeliveryDAO dao = (OrderDeliveryDAO) getDaoFactory().createDAO(
				"orderDeliveryDAOImplementer");
		dao.setOrderDelivery(orderDelivery);
		dao.updateOrderDelivery(true);
		results.put("updatedRecord", dao.getOrderDelivery());
		return results;
	}

	public Map postChangingOrderDeliveryEvent(OrderDelivery orderDelivery, Map parameters) throws Exception {
		Map results = new HashMap();
		ChangingOrderDeliveryEvent event = getBusinessObjectFactory().createObject(ChangingOrderDeliveryEvent.class);
		event.setBusinessProcessor(this);
		event.setOrderDelivery(orderDelivery);
		event.setIncomingParameters(parameters);
		event.setResults(results);
		getEventBus().post(event);
		return event.getResults();
	}

	public Map editDeleteOrderDeliveries(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple order deliveries");
		OrderDeliveryGatewayDAO dao = (OrderDeliveryGatewayDAO) getDaoFactory().createDAO(
				"orderDeliveryGatewayDAOImplementer");
		Map results = dao.editDeleteOrderDeliveries(parameters, getBusinessObjectFactory());
		return results;
	}

	public void deleteOrderDelivery(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("address1");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting order delivery: " + code);
		}
		OrderDelivery orderDelivery =  getBusinessObjectFactory()
				.createObject(OrderDelivery.class);
		BeanUtils.copyProperties(orderDelivery, parameters);
		OrderDeliveryDAO dao = (OrderDeliveryDAO) getDaoFactory().createDAO(
				"orderDeliveryDAOImplementer");
		dao.setOrderDelivery(orderDelivery);
		dao.deleteOrderDelivery(true);
		return;
	}
}
