package com.softslate.commerce.businessobjects.order;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.softslate.commerce.customer.core.FormUtils;
import com.softslate.emailer.businessobjects.core.BasicEmailerProcessor;

public class ConstantContactProcessor extends BasicEmailerProcessor {

	static Log log = LogFactory.getLog(ConstantContactProcessor.class);

	public Map register(Map parameters) throws Exception {
		return null;
	}

	public Map createClient(Map parameters) throws Exception {
		return null;
	}

	public Map updateAccessAndBilling(Map parameters) throws Exception {
		return null;
	}

	public Map deleteClient(Map parameters) throws Exception {
		return null;
	}

	public Map processContactForm(Map parameters) throws Exception {

		// Grab contact form settings for the behavior and the list name
		String behavior = (String) getSettings().getValue("constantContactContactForm");
		String listID = (String) getSettings().getValue("constantContactContactFormListID");

		// Process the integration
		return processIntegration(behavior, listID, parameters);
	}

	public Map processCustomerRegistration(Map parameters) throws Exception {

		// Grab customer registration settings for the behavior and the list
		// name
		String behavior = (String) getSettings().getValue("constantContactCustomerRegistration");
		String listID = (String) getSettings().getValue("constantContactCustomerRegistrationListID");

		// Process the integration
		return processIntegration(behavior, listID, parameters);
	}

	public Map processCheckoutAddresses(Map parameters) throws Exception {

		// Grab order placement settings for the behavior and the list name
		String behavior = (String) getSettings().getValue("constantContactCheckout");
		String listID = (String) getSettings().getValue("constantContactCheckoutListID");

		// Process the integration
		return processIntegration(behavior, listID, parameters);
	}

	public Map processIntegration(String behavior, String listID, Map parameters) throws Exception {

		if (log.isDebugEnabled())
			log.debug("parameters: " + parameters);
		// Check to see if we should add this contact
		String emailerOptInApproval = (String) parameters.get("emailerOptInApproval");
		boolean optedIn = emailerOptInApproval != null && emailerOptInApproval.equals("Yes");
		String email = (String) parameters.get("Email");
		FormUtils formUtils = (FormUtils) getBusinessObjectFactory().createUtilsObject("formUtilsImplementer");
		boolean validEmail = email != null && formUtils.isValidEmail(email);
		if (behavior.equals("dontSubscribe") || !validEmail || !(behavior.equals("subscribeAll") || optedIn)) {
			if (log.isInfoEnabled()) {
				log.info("Not adding contact: email=" + validEmail + " behavior=" + behavior + " optedIn=" + optedIn);
			}
			return null;
		}

		if (behavior.equals("subscribeAll") || (behavior.equals("subscribeOptIns") && optedIn)) {

			// Set the contact's properties
			EmailerSubscriber contact = (EmailerSubscriber) getBusinessObjectFactory().createObject(
					"emailerSubscriberImplementer");
			PropertyUtils.copyProperties(contact, parameters);
			contact.setEmail((String) parameters.get("Email"));
			contact.setFirstName((String) parameters.get("First Name"));
			contact.setLastName((String) parameters.get("Last Name"));
			contact.setHomePhone((String) parameters.get("billingPhone1"));
			((ConstantContactContact) contact).setOptedIn(optedIn);
			addContactToList(contact, listID);
		}

		Map result = new HashMap();
		result.put("Code", "0");
		result.put("Message", "Success");
		return result;
	}

	public Map addContactToList(EmailerSubscriber contact, String listID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Adding " + contact.getEmail() + " to listID " + listID);
		String responseString = "";

		// Look up to see if the contact already exists
		String existingContactID = lookupContact(contact);

		if (existingContactID != null) {

			// Contact exists. Just add him to the list then
			responseString = addExistingContactToList(contact, listID);
		} else {

			// Contact does not exist. Submit a request to CC to add the
			// contact and add him to the list in one shot.
			responseString = createContactAndAddToList(contact, listID);
		}

		Map responseMap = new HashMap();
		responseMap.put("responseString", responseString);
		return responseMap;
	}

	public String lookupContact(EmailerSubscriber contact) throws Exception {

		// A GET request to try to retrieve the ID for the contact, from his
		// email address.
		if (log.isInfoEnabled())
			log.info("Looking up to see if " + contact.getEmail() + " exists with Constant Contact.");
		String constantContactUserName = (String) getSettings().getValue("constantContactUserName");
		String email = contact.getEmail();
		String urlString = "https://api.constantcontact.com/ws/customers/" + constantContactUserName
				+ "/contacts?email=" + URLEncoder.encode(email, "UTF-8");
		if (log.isInfoEnabled())
			log.info("urlString: " + urlString);
		String responseData = sendHTTPRequest(urlString, "GET", null, false);
		if (log.isInfoEnabled())
			log.info("responseData: " + responseData);
		if (responseData.startsWith("Error code")) {
			return null;
		}
		Map results = parseResponseData(responseData);
		if (log.isDebugEnabled())
			log.debug("results: " + results);
		if (results != null && results.get("id") != null) {

			// The contact's id is the last element in the id URI that's
			// returned.
			String idElement = results.get("id").toString();
			String[] idPieces = idElement.split("/");
			String contactID = idPieces[idPieces.length - 1];
			if (!contactID.equals("contacts")) {
				if (log.isInfoEnabled())
					log.info("Contact found with id " + contactID);
				contact.setContactID(contactID);

				// Now retrieve the contact's complete information now and store
				// in the Xml property, so it can used to send updates.
				urlString = "https://api.constantcontact.com/ws/customers/" + constantContactUserName + "/contacts/"
						+ contactID;
				if (log.isInfoEnabled())
					log.info("urlString: " + urlString);
				responseData = sendHTTPRequest(urlString, "GET", null, false);
				if (log.isInfoEnabled())
					log.info("responseData: " + responseData);
				((ConstantContactContact) contact).setXml(responseData);
				return contactID;
			}
		}
		return null;
	}

	public String addExistingContactToList(EmailerSubscriber contact, String listID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Adding existing contact " + contact.getContactID() + " to list " + listID);
		String constantContactUserName = (String) getSettings().getValue("constantContactUserName");
		String optinString = "ACTION_BY_CUSTOMER";
		if (((ConstantContactContact) contact).getOptedIn()) {
			optinString = "ACTION_BY_CONTACT";
		}
		String urlString = "https://api.constantcontact.com/ws/customers/" + constantContactUserName + "/contacts/"
				+ contact.getContactID();
		if (log.isInfoEnabled())
			log.info("urlString: " + urlString);

		// Grab the xml representation for the contact that was retrieved and
		// append the new list to it.
		String postData = ((ConstantContactContact) contact).getXml();
		if (postData != null && postData.indexOf("<ContactLists>") > -1) {

			// If there's already another list
			postData = postData.replaceFirst("<ContactLists>", "<ContactLists>"
					+ "<ContactList id=\"https://api.constantcontact.com/ws/customers/" + constantContactUserName
					+ "/lists/" + listID + "\"><OptInSource>" + optinString + "</OptInSource></ContactList>");
		} else if (postData.indexOf("<ContactLists>") == -1) {

			// If this is the first/only list, create the <ContactLists> element
			// around the new list.
			postData = postData.replaceFirst("</Contact>", "<ContactLists>"
					+ "<ContactList id=\"https://api.constantcontact.com/ws/customers/" + constantContactUserName
					+ "/lists/" + listID + "\"><OptInSource>" + optinString + "</OptInSource></ContactList>"
					+ "</ContactLists></Contact>");
		}
		if (log.isInfoEnabled())
			log.info("postData: " + postData);

		// Send the request
		String responseData = sendHTTPRequest(urlString, "PUT", postData, false);

		// We return the raw response, but we actually don't care if it worked
		// or not, other than to log it.
		if (log.isInfoEnabled())
			log.info("responseData: " + responseData);
		return responseData;
	}

	public String createContactAndAddToList(EmailerSubscriber contact, String listID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Creating contact " + contact.getEmail() + " and adding to list " + listID);
		String constantContactUserName = (String) getSettings().getValue("constantContactUserName");
		String urlString = "https://api.constantcontact.com/ws/customers/" + constantContactUserName + "/contacts";
		String optinString = "ACTION_BY_CUSTOMER";
		if (((ConstantContactContact) contact).getOptedIn()) {
			optinString = "ACTION_BY_CONTACT";
		}
		if (log.isInfoEnabled())
			log.info("urlString: " + urlString);
		String postData = "<entry xmlns=\"http://www.w3.org/2005/Atom\">" + "<id>data:,</id>" + "<title/>"
				+ "<author/>" + "<updated>2010-01-01</updated>" + "<summary type=\"text\">Contact</summary>"
				+ "<content type=\"application/vnd.ctct+xml\">"
				+ "  <Contact xmlns=\"http://ws.constantcontact.com/ns/1.0/\">" + "    <EmailAddress>"
				+ contact.getEmail()
				+ "</EmailAddress>"
				+ "    <FirstName>"
				+ formatForNull(contact.getFirstName())
				+ "</FirstName>"
				+ "    <LastName>"
				+ formatForNull(contact.getLastName())
				+ "</LastName>"
				+ "    <HomePhone>"
				+ formatForNull(contact.getHomePhone())
				+ "</HomePhone>"
				+ "    <OptInSource>"
				+ optinString
				+ "</OptInSource>"
				+ "    <ContactLists>"
				+ "      <ContactList id=\"http://api.constantcontact.com/ws/customers/"
				+ constantContactUserName
				+ "/lists/" + listID + "\" />" + "    </ContactLists>" + "  </Contact>" + "</content>" + "</entry>";
		if (log.isInfoEnabled())
			log.info("postData: " + postData);

		// Send the request
		String responseData = sendHTTPRequest(urlString, "POST", postData, false);

		// We return the raw response, but we actually don't care if it worked
		// or not, other than to log it.
		if (log.isInfoEnabled())
			log.info("responseData: " + responseData);
		return responseData;
	}

	public String sendHTTPRequest(String urlString, String method, String postData, boolean followRedirect)
			throws Exception {

		// Authentication settings
		String constantContactAPIKey = (String) getSettings().getValue("constantContactAPIKey");
		String constantContactUserName = (String) getSettings().getValue("constantContactUserName");
		String constantContactPassword = (String) getSettings().getValue("constantContactPassword");

		// Set up a url connection with Basic Authentication headers
		URL url = new URL(urlString);
		HttpURLConnection con = (HttpURLConnection) url.openConnection();
		String basicAuthUser = constantContactAPIKey + "%" + constantContactUserName;
		String basicAuthPassword = constantContactPassword;
		String userpassword = basicAuthUser + ":" + basicAuthPassword;
		byte[] encoded = Base64.encodeBase64(userpassword.getBytes());
		String encodedAuthorization = new String(encoded);
		con.setRequestProperty("Authorization", "Basic " + encodedAuthorization);
		con.setRequestProperty("Content-type", "application/atom+xml");
		con.setInstanceFollowRedirects(followRedirect);
		con.setDoOutput(true);
		con.setDoInput(true);
		con.setUseCaches(false);

		// Send the request
		StringBuffer response = new StringBuffer();
		BufferedReader rd = null;
		OutputStreamWriter wr = null;
		if (postData == null || method == null || method.toUpperCase().equals("GET")) {
			con.setRequestMethod("GET");
			con.connect();
		} else if (postData != null && method.toUpperCase().equals("POST")) {
			con.setRequestMethod("POST");
			wr = new OutputStreamWriter(con.getOutputStream());
			wr.write(postData);
			wr.flush();
		} else if (postData != null && method.toUpperCase().equals("PUT")) {
			con.setRequestMethod("PUT");
			wr = new OutputStreamWriter(con.getOutputStream());
			wr.write(postData);
			wr.flush();
		} else {
			log.info("Unsupported method.");
		}

		// Capture the response
		String responseString = "";
		try {
			rd = new BufferedReader(new InputStreamReader(con.getInputStream()));
			String line;
			while ((line = rd.readLine()) != null) {
				response.append(line);
			}
			responseString = response.toString();
		} catch (IOException e) {
			if (con.getResponseCode() == 400) {
				responseString = "Error code 400: Invalid request.";
			} else if (con.getResponseCode() == 401) {
				responseString = "Error code 401: Unauthorized.";
			} else if (con.getResponseCode() == 403) {
				responseString = "Error code 403: Forbidden. (Might be you need to use https not http.)";
			} else if (con.getResponseCode() == 404) {
				responseString = "Error code 404: URL not found.";
			} else if (con.getResponseCode() == 409) {
				responseString = "Error code 409: Conflict. (This error is normal if the user is already subscribed.)";
			} else if (con.getResponseCode() == 415) {
				responseString = "Error code 415: Unsupported Media Type.";
			} else if (con.getResponseCode() == 500) {
				responseString = "Error code 500: Server Error.";
			} else {
				throw e;
			}
			responseString = responseString + " For more information see http://community.constantcontact.com/t5/"
					+ "Documentation/Error-Codes/ba-p/25077";
		} finally {
			if (wr != null) {
				wr.close();
			}
			if (rd != null) {
				rd.close();
			}
		}
		return responseString;
	}

	public Map parseResponseData(String response) throws Exception {

		Map responseMap = new HashMap();

		Document doc = null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		InputSource is = new InputSource(new StringReader(response));
		try {
			doc = builder.parse(is);
		} catch (SAXException se) {

			// If the response is not parseable, set the error message to be the
			// entire response. Workaround for their not providing certain
			// errors in XML format.
			responseMap.put("Code", "500");
			responseMap.put("Message", response);
			return responseMap;
		}

		Node n = doc.getDocumentElement().getFirstChild();
		while (n != null) {
			if (n.getFirstChild() != null) {
				responseMap.put(n.getNodeName(), n.getFirstChild().getNodeValue());
				if (n.getNodeName().equals("entry")) {
					Node child = n.getFirstChild();
					while (child != null) {
						if (child.getFirstChild() != null) {
							responseMap.put(child.getNodeName(), child.getFirstChild().getNodeValue());
						}
						child = child.getNextSibling();
					}
				}
			}
			n = n.getNextSibling();
		}

		return responseMap;
	}

	public String formatForNull(String value) {
		if (value == null) {
			return "";
		}
		return value;
	}

	/* Currently unused */
	protected String lookupLists() throws Exception {
		String constantContactUserName = (String) getSettings().getValue("constantContactUserName");
		String urlString = "https://api.constantcontact.com/ws/customers/" + constantContactUserName + "/lists";
		if (log.isInfoEnabled())
			log.info("urlString: " + urlString);
		String responseData = sendHTTPRequest(urlString, "GET", null, false);
		if (log.isInfoEnabled())
			log.info("responseData: " + responseData);
		if (responseData.startsWith("Error code")) {
			return null;
		}
		Map results = parseResponseData(responseData);
		if (log.isInfoEnabled())
			log.info("results: " + results);
		return null;
	}

}
