package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with countries. Created in
 * the Struts layer in response to requests for viewing, editing, adding, and
 * deleting countries.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CountryProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "countryProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "countryProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.CountryProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface CountryProcessor extends BusinessProcessor {

	abstract Map loadCountriesAndCount(Map parameters) throws Exception;

	abstract Map addCountry(Map parameters) throws Exception;

	abstract Country loadCountryFromID(Map parameters) throws Exception;

	abstract Map editCountry(Map parameters) throws Exception;

	abstract Map editDeleteCountries(Map parameters) throws Exception;

	abstract void deleteCountry(Map parameters) throws Exception;
}
