package com.softslate.commerce.businessobjects.order;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.shipping.ShippingRule;

/**
 * Object representing a order shipping rule within the system.
 * <code>OrderShippingRuleBean</code> is the default implementation of the
 * <code>OrderShippingRule</code> interface for the application.
 * <p>
 * The database stores order shipping rules in the
 * <code>sscOrderShippingRule</code> table, and this interface can be used to
 * represent a single row of that table. The Hibernate configuration file
 * <code>OrderShippingRule.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author Jason McWilliams
 * 
 */
@Audited
@AuditTable("AudOrderShippingRule")
public class OrderShippingRuleBean extends BaseBusinessObject implements
		OrderShippingRule, Serializable {

	private static final long serialVersionUID = -2898926399058085322L;

	private int orderShippingRuleID = 0;

	private int orderID = 0;

	private Integer orderItemID = null;

	private Double total = new Double(0);

	private Integer shippingRuleID = new Integer(0);

	private String code = null;

	private String name = null;

	private String ruleType = null;

	private Double floor = new Double(0);

	private Double ceiling = new Double(0);

	private String discountType = null;

	private Double discount = new Double(0);

	private String pkgDescription = null;

	private Double pkgWeight = new Double(0);

	private Double pkgLength = new Double(0);

	private Double pkgHeight = new Double(0);

	private Double pkgWidth = new Double(0);

	private String limitMethods = null;

	private String limitType = null;

	private String extra1 = null;

	private String extra2 = null;

	private String extra3 = null;

	private String pkgExtra1 = null;

	private String pkgExtra2 = null;

	private String pkgExtra3 = null;

	private String pkgExtra4 = null;

	private String pkgExtra5 = null;

	private Double subtotal = new Double(0);

	private Order order = null;

	private OrderItem orderItem = null;

	private ShippingRule shippingRule = null;

	private String countries = null;

	private String states = null;

	public int getOrderShippingRuleID() {
		return orderShippingRuleID;
	}

	public void setOrderShippingRuleID(int orderShippingRuleID) {
		this.orderShippingRuleID = orderShippingRuleID;
	}

	public int getOrderID() {
		return orderID;
	}

	public void setOrderID(int orderID) {
		this.orderID = orderID;
	}

	public Integer getOrderItemID() {
		return orderItemID;
	}

	public void setOrderItemID(Integer orderItemID) {
		this.orderItemID = orderItemID;
	}

	public Double getTotal() {
		return total;
	}

	public void setTotal(Double total) {
		this.total = total;
	}

	public Integer getShippingRuleID() {
		return shippingRuleID;
	}

	public void setShippingRuleID(Integer shippingRuleID) {
		this.shippingRuleID = shippingRuleID;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getRuleType() {
		return ruleType;
	}

	public void setRuleType(String ruleType) {
		this.ruleType = ruleType;
	}

	public Double getFloor() {
		return floor;
	}

	public void setFloor(Double floor) {
		this.floor = floor;
	}

	public Double getCeiling() {
		return ceiling;
	}

	public void setCeiling(Double ceiling) {
		this.ceiling = ceiling;
	}

	public String getDiscountType() {
		return discountType;
	}

	public void setDiscountType(String discountType) {
		this.discountType = discountType;
	}

	public Double getDiscount() {
		return discount;
	}

	public void setDiscount(Double discount) {
		this.discount = discount;
	}

	public String getPkgDescription() {
		return pkgDescription;
	}

	public void setPkgDescription(String pkgDescription) {
		this.pkgDescription = pkgDescription;
	}

	public Double getPkgWeight() {
		return pkgWeight;
	}

	public void setPkgWeight(Double pkgWeight) {
		this.pkgWeight = pkgWeight;
	}

	public Double getPkgLength() {
		return pkgLength;
	}

	public void setPkgLength(Double pkgLength) {
		this.pkgLength = pkgLength;
	}

	public Double getPkgHeight() {
		return pkgHeight;
	}

	public void setPkgHeight(Double pkgHeight) {
		this.pkgHeight = pkgHeight;
	}

	public Double getPkgWidth() {
		return pkgWidth;
	}

	public void setPkgWidth(Double pkgWidth) {
		this.pkgWidth = pkgWidth;
	}

	public String getLimitMethods() {
		return limitMethods;
	}

	public void setLimitMethods(String limitMethods) {
		this.limitMethods = limitMethods;
	}

	public String getLimitType() {
		return limitType;
	}

	public void setLimitType(String limitType) {
		this.limitType = limitType;
	}

	public String getExtra1() {
		return extra1;
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public String getExtra2() {
		return extra2;
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public String getPkgExtra1() {
		return pkgExtra1;
	}

	public void setPkgExtra1(String pkgExtra1) {
		this.pkgExtra1 = pkgExtra1;
	}

	public String getPkgExtra2() {
		return pkgExtra2;
	}

	public void setPkgExtra2(String pkgExtra2) {
		this.pkgExtra2 = pkgExtra2;
	}

	public String getPkgExtra3() {
		return pkgExtra3;
	}

	public void setPkgExtra3(String pkgExtra3) {
		this.pkgExtra3 = pkgExtra3;
	}

	public String getPkgExtra4() {
		return pkgExtra4;
	}

	public void setPkgExtra4(String pkgExtra4) {
		this.pkgExtra4 = pkgExtra4;
	}

	public String getPkgExtra5() {
		return pkgExtra5;
	}

	public void setPkgExtra5(String pkgExtra5) {
		this.pkgExtra5 = pkgExtra5;
	}

	public Double getSubtotal() {
		return subtotal;
	}

	public void setSubtotal(Double subtotal) {
		this.subtotal = subtotal;
	}

	public ShippingRule getShippingRule() {
		return shippingRule;
	}

	public void setShippingRule(ShippingRule shippingRule) {
		this.shippingRule = shippingRule;
	}

	public Order getOrder() {
		return order;
	}

	public void setOrder(Order order) {
		this.order = order;
	}

	public OrderItem getOrderItem() {
		return orderItem;
	}

	public void setOrderItem(OrderItem orderItem) {
		this.orderItem = orderItem;
	}

	public String getCountries() {
		return countries;
	}

	public void setCountries(String countries) {
		this.countries = countries;
	}

	public String getStates() {
		return states;
	}

	public void setStates(String states) {
		this.states = states;
	}

}
