package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.daos.payment.PaymentDAO;

/**
 * Processes a purchase order payment for the store. Simply records the fact
 * that the user elected to by pay with a purchase order, so that the order can
 * be looked up and processed after payment is received.
 * 
 * @author Jason McWilliams
 * @see com.softslate.commerce.daos.payment.PaymentDAO
 */
public class PurchaseOrderPaymentProcessor extends BasePaymentProcessor
		implements PaymentProcessor {

	static Log log = LogFactory.getLog(PurchaseOrderPaymentProcessor.class);

	public Map process(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing payment.");
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		BeanUtils.copyProperties(payment, parameters);
		payment.setStatus("Pending");
		payment.setOrder(getUser().getOrder());
		payment.setProcessorClassName(this.getClass().getName());

		String purchaseOrderNumber = (String) parameters
				.get("purchaseOrderNumber");
		if (purchaseOrderNumber == null || purchaseOrderNumber.length() < 1) {
			purchaseOrderNumber = "0";
		}

		String purchaseOrderNotes = (String) parameters
				.get("purchaseOrderNotes");

		payment.setValueFieldLabels(",,,Details,PO,,,,Customer Notes");
		payment.setAmount(getUser().getOrder().getTotal());
		payment.setCreated(formatDateTime(new java.util.Date()));
		payment.setValue4("PO: " + purchaseOrderNumber);
		payment.setValue5(purchaseOrderNumber);
		payment.setValue9(purchaseOrderNotes);
		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		paymentDAO.setPayment(payment);
		paymentDAO.insertPayment(false);
		if (getUser().getOrder().getPayments() == null) {
			getUser().getOrder().setPayments(new LinkedHashSet());
		}
		getUser().getOrder().getPayments().add(payment);

		Map result = new HashMap();
		result.put("newPayment", payment);
		result.put("resultCode", "0");
		result.put("resultMessage","Payment successful");
		return result;
	}
}
