package com.softslate.commerce.businessobjects.payment.authorizeNetCIM;

import java.io.StringReader;
import java.io.StringWriter;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import AuthNet.Rebill.ArrayOfCustomerAddressType;
import AuthNet.Rebill.ArrayOfCustomerPaymentProfileType;
import AuthNet.Rebill.BankAccountType;
import AuthNet.Rebill.CreateCustomerPaymentProfileResponseType;
import AuthNet.Rebill.CreateCustomerProfileResponseType;
import AuthNet.Rebill.CreateCustomerProfileTransactionResponseType;
import AuthNet.Rebill.CreditCardType;
import AuthNet.Rebill.CustomerAddressType;
import AuthNet.Rebill.CustomerPaymentProfileExType;
import AuthNet.Rebill.CustomerPaymentProfileMaskedType;
import AuthNet.Rebill.CustomerPaymentProfileType;
import AuthNet.Rebill.CustomerProfileType;
import AuthNet.Rebill.DeleteCustomerPaymentProfileResponseType;
import AuthNet.Rebill.DriversLicenseType;
import AuthNet.Rebill.ExtendedAmountType;
import AuthNet.Rebill.GetCustomerProfileResponseType;
import AuthNet.Rebill.LineItemType;
import AuthNet.Rebill.MessageTypeEnum;
import AuthNet.Rebill.OrderExType;
import AuthNet.Rebill.PaymentType;
import AuthNet.Rebill.ProfileTransAuthCaptureType;
import AuthNet.Rebill.ProfileTransAuthOnlyType;
import AuthNet.Rebill.ProfileTransPriorAuthCaptureType;
import AuthNet.Rebill.ProfileTransVoidType;
import AuthNet.Rebill.ProfileTransactionType;
import AuthNet.Rebill.ServiceSoap;
import AuthNet.Rebill.UpdateCustomerPaymentProfileResponseType;
import AuthNet.Rebill.ValidationModeEnum;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.payment.BasePaymentProcessor;
import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.businessobjects.payment.PaymentProcessor;
import com.softslate.commerce.businessobjects.payment.authorizeNet.AuthorizeNetAIMConstants;
import com.softslate.commerce.customer.order.BillingAddressForm;
import com.softslate.commerce.daos.customer.CustomerGatewayDAO;
import com.softslate.commerce.daos.payment.PaymentDAO;
import com.softslate.commerce.daos.payment.PaymentGatewayDAO;

/**
 * Handles Authorize.net CIM profiles and payments
 * 
 * @author Jason McWilliams
 * @see com.softslate.commerce.daos.payment.PaymentDAO
 */
public class AuthorizeNetCIMProcessorImpl extends BasePaymentProcessor implements PaymentProcessor,
		AuthorizeNetCIMProcessor {

	static Log log = LogFactory.getLog(AuthorizeNetCIMProcessor.class);

	public String TEST_MODE;
	public String WSDL_URL;
	public String SERVICE_URL;
	public String VALIDATION_MODE;
	public String MERCHANT_NAME;
	public String TRANSACTION_KEY;

	public Map process(Map parameters) throws Exception {
		log.info("Processing payment.");
		initialize();
		return processAuthorizeNetCIMPayment(parameters);
	}

	public Map processAuthorizeNetCIMPayment(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("processAuthorizeNetCIMPayment");

		Map result = new HashMap();
		Order order = null;
		Customer customer = null;
		long useProfileID = 0;
		long cimPaymentProfileID = 0;
		String merchantCustomerID = null;
		String cimPaymentProfileIDStr = (String) parameters.get("cimPaymentProfileID");
		String cimPaymentProfileType = (String) parameters.get("cimPaymentProfileType");

		try {
			cimPaymentProfileID = Long.parseLong(cimPaymentProfileIDStr);
		} catch (Exception ex) {
			cimPaymentProfileID = 0;
		}

		if (cimPaymentProfileID < 1) {
			result.put("resultCode", "90");
			result.put("resultMessage", null);
			return result;
		}

		if (!"order".equalsIgnoreCase(cimPaymentProfileType) && !"customer".equalsIgnoreCase(cimPaymentProfileType)) {
			result.put("resultCode", "90");
			result.put("resultMessage", null);
			return result;
		}

		if (getUser() != null && getUser().getOrder() != null) {
			order = getUser().getOrder();
		}
		if (getUser() != null && getUser().getCustomer() != null) {
			customer = getUser().getCustomer();
		}
		if (order == null) {
			return null;
		}

		if ("customer".equalsIgnoreCase(cimPaymentProfileType)) {
			if (StringUtils.isNotBlank(customer.getCimProfileID())) {
				useProfileID = Long.parseLong(customer.getCimProfileID());
				merchantCustomerID = "c-" + customer.getCustomerID();
			}
		} else {
			if (StringUtils.isNotBlank(order.getPaymentProfileID())) {
				useProfileID = Long.parseLong(order.getPaymentProfileID());
				merchantCustomerID = "o-" + order.getOrderID();
			}
		}
		if (useProfileID < 1) {
			result.put("resultCode", "91");
			result.put("resultMessage", null);
			return result;
		}

		// Security checks to be sure we are using the correct profile and
		// everything matches up before billing.
		AuthorizeNetCIMResultGetProfile getProfileResult = getCustomerProfile(useProfileID);
		if (getProfileResult.getError()) {
			logProfileErrors("Error getting profile inforamtion.", getProfileResult.getMessages(), customer, order,
					"getCustomerProfile");
			result.put("resultCode", "92");
			result.put("resultMessage", null);
			return result;

		} else {
			if (getProfileResult.getProfileID() > 0 && getProfileResult.getProfileID() == useProfileID
					&& merchantCustomerID.equals(getProfileResult.getMerchantCustomerID())) {

				boolean foundMatchingPayment = false;
				if (getProfileResult.getPaymentProfiles() != null && getProfileResult.getPaymentProfiles().size() > 0) {
					for (AuthorizeNetCIMPaymentProfileMin profile : getProfileResult.getPaymentProfiles()) {
						if (profile.getCustomerPaymentProfileId() > 0
								&& profile.getCustomerPaymentProfileId() == cimPaymentProfileID) {
							foundMatchingPayment = true;
							break;
						}
					}
				}
				if (!foundMatchingPayment) {
					logProfileErrors("Cant find matching profile for chosen profile.", getProfileResult.getMessages(),
							customer, order, "getCustomerProfile");
					result.put("resultCode", "93");
					result.put("resultMessage", null);
					return result;
				}
			} else {
				// Not a good thing here
				logProfileErrors(
						"Returned profileID, merchantCustomerID does not match the sent profileID, merchantCustomerID",
						getProfileResult.getMessages(), customer, order, "getCustomerProfile");
				result.put("resultCode", "93");
				result.put("resultMessage", null);
				return result;
			}
		}

		// Good to try to charge.
		String authorizeNetCIMTxType = (String) getSettings().getValue("authorizeNetCIMTxType");
		AuthorizeNetCIMResult transactionResult = null;

		if ("AUTH_ONLY".equals(authorizeNetCIMTxType)) {
			transactionResult = processAuthOnly(order, parameters, useProfileID, cimPaymentProfileID, 0);
		} else if ("AUTH_CAPTURE".equals(authorizeNetCIMTxType)) {
			transactionResult = processAuthCapture(order, parameters, useProfileID, cimPaymentProfileID, 0);
		} else {
			transactionResult = processAuthCapture(order, parameters, useProfileID, cimPaymentProfileID, 0);
		}

		if (transactionResult.getError()) {
			Map<String, String> error = logProfileErrors("Authorize.net transaction errors",
					transactionResult.getMessages(), customer, order, "processingPayment");

			String resultCode = null;
			String resultMessage = null;
			if (error != null) {
				resultCode = error.get("code");
				resultMessage = error.get("text");
			}
			if (resultCode == null) {
				resultCode = "XX";
			}
			result.put("resultCode", resultCode);
			result.put("resultMessage", resultMessage);
			return result;
		}
		transactionResult.setProfileID(useProfileID);
		transactionResult.setPaymentProfileID(cimPaymentProfileID);

		// Must be a success
		return processSuccessResponse(parameters, transactionResult, false);
	}

	public AuthorizeNetCIMResult processAuthOnly(Order order, Map parameters, long customerProfileId,
			long customerPaymentProfileId, long customerShippingAddressId) throws Exception {
		log.info("Processing auth only.");

		ProfileTransactionType trans = new ProfileTransactionType();
		ProfileTransAuthOnlyType authOnly = new ProfileTransAuthOnlyType();
		authOnly.setAmount(new BigDecimal(order.getTotal().toString()));

		if (customerProfileId > 0) {
			authOnly.setCustomerProfileId(customerProfileId);
		}
		if (customerPaymentProfileId > 0) {
			authOnly.setCustomerPaymentProfileId(customerPaymentProfileId);
		}
		if (customerShippingAddressId > 0) {
			authOnly.setCustomerShippingAddressId(customerShippingAddressId);
		}

		authOnly.setOrder(buildOrderExType(parameters));

		ExtendedAmountType shipping = new ExtendedAmountType();
		if (order.getShipping() != null) {
			shipping.setAmount(new BigDecimal(order.getShipping().toString()));
			shipping.setName(fixString(order.getShippingMethod(), 31));
			authOnly.setShipping(shipping);
		}

		ExtendedAmountType tax = new ExtendedAmountType();
		if (order.getTax() != null) {
			tax.setAmount(new BigDecimal(order.getTax().toString()));
			authOnly.setTax(tax);
		}

		String authorizeNetCIMIncludeLineItems = (String) getSettings().getValue("authorizeNetCIMIncludeLineItems");
		if (authorizeNetCIMIncludeLineItems != null && authorizeNetCIMIncludeLineItems.equals("1")) {
			Collection<OrderItem> orderItems = null;
			if (order.firstDelivery() != null) {
				orderItems = order.firstDelivery().getOrderItemCollection();
			}
			buildTransactionLineItems(authOnly.getLineItems(), orderItems);
		}

		// START NOT IMPLEMENTED YET
		// authCapture.setTaxExempt(value);
		// authCapture.setRecurringBilling(value);
		// authCapture.setDuty(value);
		// authCapture.setCardCode(value);

		// END NOT IMPLEMENTED YET

		// x_test_request can't be used because CIM API will not run in test
		// mode, otherwise we could put it in the extraOptions here if it does
		// sometime.

		trans.setProfileTransAuthOnly(authOnly);

		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);

		CreateCustomerProfileTransactionResponseType response = soap.createCustomerProfileTransaction(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), trans,
				buildExtraOptions());

		AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();

		boolean error = false;
		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
			authorizeNetCIMResult.setError(true);
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
			authorizeNetCIMResult.setDirectResponse(response.getDirectResponse());

		}

		return authorizeNetCIMResult;

	}

	public AuthorizeNetCIMResult processPriorAuthCapture(Order order, Map parameters) throws Exception {
        log.info("Processing capture of prior auth");
		initialize();
        ProfileTransactionType trans = new ProfileTransactionType();
        ProfileTransPriorAuthCaptureType authCapture = new ProfileTransPriorAuthCaptureType();
        authCapture.setAmount(new BigDecimal(order.getTotal().toString()));
        authCapture.setTransId((String)parameters.get("authorizeTransID"));
        trans.setProfileTransPriorAuthCapture(authCapture);

        ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);

        CreateCustomerProfileTransactionResponseType response = soap.createCustomerProfileTransaction(
                AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), trans,
                buildExtraOptions());

        AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();

        boolean error = false;
        if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
            error = true;
            authorizeNetCIMResult.setError(true);
        }

        if (response != null) {
            authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
            authorizeNetCIMResult.setDirectResponse(response.getDirectResponse());

        }

        return authorizeNetCIMResult;
    }

    /**
     * This method is called from the events system when the setting
     * "Order Status for Capturing Payments" is set and matches the event.
     */
    public Map processPaymentCapture(Order order) throws Exception {
        log.info("Processing AuthorizeNetCIM processPaymentCapture");

        Map result = new HashMap();
        Customer customer = null;
        if (getUser() != null && getUser().getCustomer() != null) {
            customer = getUser().getCustomer();
        }

        Collection<Payment> payments = order.getPayments();
        if (payments != null && payments.size() > 0) {
            // Check to see if we have a payment for this order that is captured
            log.info("Looking to see if we have a captured payment");
            for (Payment payment : payments) {

				// Auth Code is in value 1.
                // Note: this assumes one auth and one capture per order.  If we need
                // to support multiple payments per order, we will need to adjust.
                if(payment.getValue1() != null && !payment.getValue1().isEmpty() &&
                        payment.getValue1().equals("prior_auth_capture")){
                    log.info("Found payment with captured code " + payment.getValue1());
					result.put("resultCode", "-1");
					result.put("resultMessage", "Found payment with captured code " + payment.getValue1());
					return result;
                }
            }

            // Check if we have a payment with an authorization code
            log.info("looking for payment with authorization code");
            for (Payment payment : payments) {

                // Auth Code is in value 6
                if(payment.getValue6() != null && !payment.getValue6().isEmpty()){
                    log.info("Found payment with authorization code " + payment.getValue6());

                    // Check if the order total is greater and throw an error if it is.
                    // Authorize will reject it anyway.  We could check if greater than
                    // 30 days too
                    if(order.getTotal() > payment.getAmount()){
						result.put("resultCode", "-2");
						result.put(
								"resultMessage",
								"Found authorized payment but amount is less than current order total: "
										+ payment.getValue1() + "; payment amount: " + payment.getAmount()
										+ "; order total: " + order.getTotal());
						return result;
                    }

                    // Capture the payment now
                    AuthorizeNetCIMResult transactionResult = null;
                    Map parameters = new HashMap();
                    parameters.put("authorizeTransID",payment.getValue7());
                    transactionResult = processPriorAuthCapture(order, parameters);
                    if (transactionResult.getError()) {
                        Map<String, String> error = logProfileErrors("Authorize.net transaction errors",
                                transactionResult.getMessages(), customer, order, "processingPayment");

                        String resultCode = null;
                        String resultMessage = null;
                        if (error != null) {
                            resultCode = error.get("code");
                            resultMessage = error.get("text");
                        }
                        if (resultCode == null) {
                            resultCode = "XX";
                        }
                        result.put("resultCode", resultCode);
                        result.put("resultMessage", resultMessage);
                        return result;
                    }

                    // Must be a success
                    if(payment.getPaymentProfileID() != null && !payment.getPaymentProfileID().equals("0")) {
                        transactionResult.setPaymentProfileID(Long.parseLong(payment.getPaymentProfileID()));
                    }
                    parameters.put("order",order);
                    return processSuccessResponse(parameters, transactionResult, true);
                }
            }
        }
		result.put("resultCode", "-3");
		result.put("resultMessage", "No authorized payment found for the order.");
		return result;
    }

	public AuthorizeNetCIMResult processAuthCapture(Order order, Map parameters, long customerProfileId,
			long customerPaymentProfileId, long customerShippingAddressId) throws Exception {
		log.info("Processing auth capture.");

		ProfileTransactionType trans = new ProfileTransactionType();
		ProfileTransAuthCaptureType authCapture = new ProfileTransAuthCaptureType();
		authCapture.setAmount(new BigDecimal(order.getTotal().toString()));

		if (customerProfileId > 0) {
			authCapture.setCustomerProfileId(customerProfileId);
		}
		if (customerPaymentProfileId > 0) {
			authCapture.setCustomerPaymentProfileId(customerPaymentProfileId);
		}
		if (customerShippingAddressId > 0) {
			authCapture.setCustomerShippingAddressId(customerShippingAddressId);
		}

		authCapture.setOrder(buildOrderExType(parameters));

		ExtendedAmountType shipping = new ExtendedAmountType();
		if (order.getShipping() != null) {
			shipping.setAmount(new BigDecimal(order.getShipping().toString()));
			shipping.setName(fixString(order.getShippingMethod(), 31));
			authCapture.setShipping(shipping);
		}

		ExtendedAmountType tax = new ExtendedAmountType();
		if (order.getTax() != null) {
			tax.setAmount(new BigDecimal(order.getTax().toString()));
			authCapture.setTax(tax);
		}

		String authorizeNetCIMIncludeLineItems = (String) getSettings().getValue("authorizeNetCIMIncludeLineItems");
		if (authorizeNetCIMIncludeLineItems != null && authorizeNetCIMIncludeLineItems.equals("1")) {
			Collection<OrderItem> orderItems = null;
			if (order.firstDelivery() != null) {
				orderItems = order.firstDelivery().getOrderItemCollection();
			}
			buildTransactionLineItems(authCapture.getLineItems(), orderItems);
		}

		// START NOT IMPLEMENTED YET
		// authCapture.setTaxExempt(value);
		// authCapture.setRecurringBilling(value);
		// authCapture.setDuty(value);
		// authCapture.setCardCode(value);

		// END NOT IMPLEMENTED YET

		// x_test_request can't be used because CIM API will not run in test
		// mode, otherwise we could put it in the extraOptions here if it does
		// sometime.

		trans.setProfileTransAuthCapture(authCapture);

		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);

		CreateCustomerProfileTransactionResponseType response = soap.createCustomerProfileTransaction(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), trans,
				buildExtraOptions());

		AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();

		boolean error = false;
		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
			authorizeNetCIMResult.setError(true);
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
			authorizeNetCIMResult.setDirectResponse(response.getDirectResponse());

		}

		return authorizeNetCIMResult;

	}

	public Map processVoid(Map parameters) throws Exception {
		log.info("Processing void transaction for transactionID " + parameters.get("transactionID"));
		initialize();
		ProfileTransactionType trans = new ProfileTransactionType();
		ProfileTransVoidType voidTransaction = new ProfileTransVoidType();
		voidTransaction.setTransId((String) parameters.get("transactionID"));
		trans.setProfileTransVoid(voidTransaction);

		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);

		CreateCustomerProfileTransactionResponseType response = soap.createCustomerProfileTransaction(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), trans,
				buildExtraOptions());

		AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();

		boolean error = false;
		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
			authorizeNetCIMResult.setError(true);
			log.error("Failed to void payment with transactionID: " + parameters.get("transactionID"));
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
			authorizeNetCIMResult.setDirectResponse(response.getDirectResponse());
		}

		Map result = PropertyUtils.describe(authorizeNetCIMResult);
		log.info("Void results: " + result);
		return result;
	}

	protected String buildExtraOptions() {
		String extraOptions = "x_delim_data=TRUE&x_delim_char=|&x_duplicate_window="
				+ (String) getSettings().getValue("authorizeNetCIMDupeTxTimeout");

		String emailCust = (String) getSettings().getValue("authorizeNetCIMEmailCustomer");
		if (emailCust != null && emailCust.equals("1")) {
			extraOptions = extraOptions + "&x_email_customer=TRUE";
		} else {
			extraOptions = extraOptions + "&x_email_customer=FALSE";
		}

		String merchantEmail = (String) getSettings().getValue("authorizeNetEmailMerchant");
		if (StringUtils.isNotBlank(merchantEmail)) {
			extraOptions = extraOptions + "&x_merchant_email=" + merchantEmail;
		}
		return extraOptions;

	}

	protected OrderExType buildOrderExType(Map parameters) {
		OrderExType orderType = new OrderExType();
		String description = (String) parameters.get("description");
		String invoiceNumber = (String) parameters.get("invoiceNumber");
		String purchaseOrderNumber = (String) parameters.get("purchaseOrderNumber");
		if (StringUtils.isNotBlank(description)) {
			orderType.setDescription(description);
		}
		if (StringUtils.isNotBlank(invoiceNumber)) {
			orderType.setInvoiceNumber(invoiceNumber);
		}
		if (StringUtils.isNotBlank(purchaseOrderNumber)) {
			orderType.setPurchaseOrderNumber(purchaseOrderNumber);
		}
		return orderType;
	}

	public AuthorizeNetCIMResult deleteCustomerProfile(long profileID) {
		log.info("Deleting CIM customer profile for cimProfileID: " + profileID);
		boolean error = false;
		AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();
		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);
		GetCustomerProfileResponseType response = soap.getCustomerProfile(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), profileID);
		if (!response.getResultCode().equals(MessageTypeEnum.OK)) {
			error = true;
		}
		authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);

		return authorizeNetCIMResult;
	}

	public AuthorizeNetCIMResultGetProfile getCustomerProfile(long profileID) {
		log.info("Getting CIM customer profile for profileID: " + profileID);
		boolean error = false;
		AuthorizeNetCIMResultGetProfile authorizeNetCIMResult = new AuthorizeNetCIMResultGetProfile();
		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);
		GetCustomerProfileResponseType response = soap.getCustomerProfile(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), profileID);
		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
			authorizeNetCIMResult.setError(true);
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error,
					response.getProfile());
		}

		return authorizeNetCIMResult;
	}
	
	/*
	 * Shocking - Authorize.net does not have this in their SOAP SDK. We have to
	 * post good old fashioned XML to get this information. Great.
	 */
	public Map<String, String> getHostedProfilePage(Long cimProfileID)
			throws Exception {

		Map<String, String> results = new HashMap<String, String>();

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		DOMImplementation impl = builder.getDOMImplementation();

		// Create access request container information
		Document document1 = impl.createDocument(
				"AnetApi/xml/v1/schema/AnetApiSchema.xsd",
				"getHostedProfilePageRequest", null);
		Element getHostedProfilePageRequest = document1.getDocumentElement();

		Element merchantAuthentication = document1
				.createElement("merchantAuthentication");
		getHostedProfilePageRequest.appendChild(merchantAuthentication);

		Element name = (Element) document1.createElement("name");
		merchantAuthentication.appendChild(name);
		name.appendChild(document1.createTextNode(MERCHANT_NAME));

		Element transactionKey = (Element) document1
				.createElement("transactionKey");
		merchantAuthentication.appendChild(transactionKey);
		transactionKey.appendChild(document1.createTextNode(TRANSACTION_KEY));

		Element customerProfileId = (Element) document1
				.createElement("customerProfileId");
		getHostedProfilePageRequest.appendChild(customerProfileId);
		customerProfileId.appendChild(document1.createTextNode(Long
				.toString(cimProfileID)));

		Element hostedProfileSettings = (Element) document1
				.createElement("hostedProfileSettings");
		getHostedProfilePageRequest.appendChild(hostedProfileSettings);

		String authorizeNetCIMUseIFrame = (String) getSettings().getValue(
				"authorizeNetCIMUseIFrame");
		boolean useIframe = false;
		if (authorizeNetCIMUseIFrame.equals("1")) {
			useIframe = true;
		}

		String authorizeNetCIMReturnUrl = (String) getSettings().getValue(
				"authorizeNetCIMReturnUrl");
		String authorizeNetCIMReturnUrlText = (String) getSettings().getValue(
				"authorizeNetCIMReturnUrlText");
		String authorizeNetCIMHeadingBgColor = (String) getSettings().getValue(
				"authorizeNetCIMHeadingBgColor");
		String authorizeNetCIMIFrameCommunicatorUrl = (String) getSettings()
				.getValue("authorizeNetCIMIFrameCommunicatorUrl");
		String authorizeNetCIMTestRequest = (String) getSettings().getValue(
				"authorizeNetCIMTestRequest");
		String authorizeNetCIMApiLiveURL = (String) getSettings().getValue(
				"authorizeNetCIMApiLiveURL");
		String authorizeNetCIMApiTestURL = (String) getSettings().getValue(
				"authorizeNetCIMApiTestURL");
		String customerSecureURL = (String) getSettings().getValue(
				"customerSecureURL");

		String authUrl = null;

		if (authorizeNetCIMTestRequest != null
				&& authorizeNetCIMTestRequest.equals("1")) {
			authUrl = authorizeNetCIMApiTestURL;
		} else {
			authUrl = authorizeNetCIMApiLiveURL;
		}

		if (!useIframe) {
			Element setting1 = (Element) document1.createElement("setting");
			hostedProfileSettings.appendChild(setting1);
			Element setting1_name = (Element) document1
					.createElement("settingName");
			Element setting1_value = (Element) document1
					.createElement("settingValue");
			setting1.appendChild(setting1_name);
			setting1.appendChild(setting1_value);
			setting1_name.appendChild(document1
					.createTextNode("hostedProfileReturnUrl"));
			setting1_value.appendChild(document1
					.createTextNode(authorizeNetCIMReturnUrl));
		}

		if (!useIframe) {
			Element setting2 = (Element) document1.createElement("setting");
			hostedProfileSettings.appendChild(setting2);
			Element setting2_name = (Element) document1
					.createElement("settingName");
			Element setting2_value = (Element) document1
					.createElement("settingValue");
			setting2.appendChild(setting2_name);
			setting2.appendChild(setting2_value);
			setting2_name.appendChild(document1
					.createTextNode("hostedProfileReturnUrlText"));
			setting2_value.appendChild(document1
					.createTextNode(authorizeNetCIMReturnUrlText));
		}

		Element setting3 = (Element) document1.createElement("setting");
		hostedProfileSettings.appendChild(setting3);
		Element setting3_name = (Element) document1
				.createElement("settingName");
		Element setting3_value = (Element) document1
				.createElement("settingValue");
		setting3.appendChild(setting3_name);
		setting3.appendChild(setting3_value);
		setting3_name.appendChild(document1
				.createTextNode("hostedProfilePageBorderVisible"));
		if (useIframe) {
			setting3_value.appendChild(document1.createTextNode("false"));
		} else {
			setting3_value.appendChild(document1.createTextNode("true"));
		}

		Element setting4 = (Element) document1.createElement("setting");
		hostedProfileSettings.appendChild(setting4);
		Element setting4_name = (Element) document1
				.createElement("settingName");
		Element setting4_value = (Element) document1
				.createElement("settingValue");
		setting4.appendChild(setting4_name);
		setting4.appendChild(setting4_value);
		setting4_name.appendChild(document1
				.createTextNode("hostedProfileHeadingBgColor"));
		setting4_value.appendChild(document1
				.createTextNode(authorizeNetCIMHeadingBgColor));

		if (useIframe) {
			Element setting5 = (Element) document1.createElement("setting");
			hostedProfileSettings.appendChild(setting5);
			Element setting5_name = (Element) document1
					.createElement("settingName");
			Element setting5_value = (Element) document1
					.createElement("settingValue");
			setting5.appendChild(setting5_name);
			setting5.appendChild(setting5_value);
			setting5_name.appendChild(document1
					.createTextNode("hostedProfileIFrameCommunicatorUrl"));
			setting5_value.appendChild(document1
					.createTextNode(customerSecureURL
							+ authorizeNetCIMIFrameCommunicatorUrl));
		}

		StringWriter output1 = new StringWriter();

		try {
			TransformerFactory.newInstance().newTransformer().transform(
					new DOMSource(document1), new StreamResult(output1));
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Error transforming XML", ex);
			}
		}

		Map<String, String> requestProperties = new HashMap<String, String>();
		requestProperties.put("Content-type", "text/xml; charset=utf-8");
		StringBuffer xmlOut;
		try {
			String responseData = sendHTTPPost(authUrl, output1.toString(),
					true, requestProperties, "utf-8");
			xmlOut = new StringBuffer(responseData);
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Error sending request to authorize.net: ", ex);
			}
			return null;
		}

		try {
			results = parseProfilePageResponseXML(xmlOut.toString());
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Error parsing XML result", ex);
			}
		}

		return results;

	}

	/*
	 * Shocking - Authorize.net does not have this in their SOAP SDK. We have to
	 * post good old fashioned XML to get this information. Great.
	 */
	public Map<String, String> getHostedProfilePage(Long cimProfileID, String fromWhere) throws Exception {
		log.info("Getting CIM hosted profile page for cimProfileID: " + cimProfileID + " from: " + fromWhere);

		Map<String, String> results = new HashMap<String, String>();

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		DOMImplementation impl = builder.getDOMImplementation();

		// Create access request container information
		Document document1 = impl.createDocument("AnetApi/xml/v1/schema/AnetApiSchema.xsd",
				"getHostedProfilePageRequest", null);
		Element getHostedProfilePageRequest = document1.getDocumentElement();

		Element merchantAuthentication = document1.createElement("merchantAuthentication");
		getHostedProfilePageRequest.appendChild(merchantAuthentication);

		Element name = (Element) document1.createElement("name");
		merchantAuthentication.appendChild(name);
		name.appendChild(document1.createTextNode(MERCHANT_NAME));

		Element transactionKey = (Element) document1.createElement("transactionKey");
		merchantAuthentication.appendChild(transactionKey);
		transactionKey.appendChild(document1.createTextNode(TRANSACTION_KEY));

		Element customerProfileId = (Element) document1.createElement("customerProfileId");
		getHostedProfilePageRequest.appendChild(customerProfileId);
		customerProfileId.appendChild(document1.createTextNode(Long.toString(cimProfileID)));

		Element hostedProfileSettings = (Element) document1.createElement("hostedProfileSettings");
		getHostedProfilePageRequest.appendChild(hostedProfileSettings);

		String authorizeNetCIMHostedMethod = (String) getSettings().getValue("authorizeNetCIMHostedMethod");
		boolean useIframe = false;
		if (!"Redirect".equalsIgnoreCase(authorizeNetCIMHostedMethod)) {
			useIframe = true;
		}
		String authorizeNetCIMReturnUrlText = (String) getSettings().getValue("authorizeNetCIMReturnUrlText");
		String authorizeNetCIMHeadingBgColor = (String) getSettings().getValue("authorizeNetCIMHeadingBgColor");
		String authorizeNetCIMIFrameCommunicatorUrl = (String) getSettings().getValue(
				"authorizeNetCIMIFrameCommunicatorUrl");
		String authorizeNetCIMTestRequest = (String) getSettings().getValue("authorizeNetCIMTestRequest");
		String authorizeNetCIMApiLiveURL = (String) getSettings().getValue("authorizeNetCIMApiLiveURL");
		String authorizeNetCIMApiTestURL = (String) getSettings().getValue("authorizeNetCIMApiTestURL");
		String customerSecureURL = (String) getSettings().getValue("customerSecureURL");

		String authUrl = null;

		if (authorizeNetCIMTestRequest != null && authorizeNetCIMTestRequest.equals("1")) {
			authUrl = authorizeNetCIMApiTestURL;
		} else {
			authUrl = authorizeNetCIMApiLiveURL;
		}

		if (!useIframe) {
			Element setting1 = (Element) document1.createElement("setting");
			hostedProfileSettings.appendChild(setting1);
			Element setting1_name = (Element) document1.createElement("settingName");
			Element setting1_value = (Element) document1.createElement("settingValue");
			setting1.appendChild(setting1_name);
			setting1.appendChild(setting1_value);
			setting1_name.appendChild(document1.createTextNode("hostedProfileReturnUrl"));
			String returnURL = "CheckoutPaymentForm.do";
			if ("account".equals(fromWhere)) {
				returnURL = "AccountCIMManageProfiles.do";
			}
			returnURL = customerSecureURL + "/" + returnURL;

			setting1_value.appendChild(document1.createTextNode(returnURL));
		}

		if (!useIframe) {
			Element setting2 = (Element) document1.createElement("setting");
			hostedProfileSettings.appendChild(setting2);
			Element setting2_name = (Element) document1.createElement("settingName");
			Element setting2_value = (Element) document1.createElement("settingValue");
			setting2.appendChild(setting2_name);
			setting2.appendChild(setting2_value);
			setting2_name.appendChild(document1.createTextNode("hostedProfileReturnUrlText"));
			setting2_value.appendChild(document1.createTextNode(authorizeNetCIMReturnUrlText));
		}

		Element setting3 = (Element) document1.createElement("setting");
		hostedProfileSettings.appendChild(setting3);
		Element setting3_name = (Element) document1.createElement("settingName");
		Element setting3_value = (Element) document1.createElement("settingValue");
		setting3.appendChild(setting3_name);
		setting3.appendChild(setting3_value);
		setting3_name.appendChild(document1.createTextNode("hostedProfilePageBorderVisible"));
		if (useIframe) {
			setting3_value.appendChild(document1.createTextNode("false"));
		} else {
			setting3_value.appendChild(document1.createTextNode("true"));
		}

		Element setting4 = (Element) document1.createElement("setting");
		hostedProfileSettings.appendChild(setting4);
		Element setting4_name = (Element) document1.createElement("settingName");
		Element setting4_value = (Element) document1.createElement("settingValue");
		setting4.appendChild(setting4_name);
		setting4.appendChild(setting4_value);
		setting4_name.appendChild(document1.createTextNode("hostedProfileHeadingBgColor"));
		setting4_value.appendChild(document1.createTextNode(authorizeNetCIMHeadingBgColor));

		if (useIframe) {
			Element setting5 = (Element) document1.createElement("setting");
			hostedProfileSettings.appendChild(setting5);
			Element setting5_name = (Element) document1.createElement("settingName");
			Element setting5_value = (Element) document1.createElement("settingValue");
			setting5.appendChild(setting5_name);
			setting5.appendChild(setting5_value);
			setting5_name.appendChild(document1.createTextNode("hostedProfileIFrameCommunicatorUrl"));
			setting5_value.appendChild(document1.createTextNode(customerSecureURL
					+ authorizeNetCIMIFrameCommunicatorUrl));
		}

		StringWriter output1 = new StringWriter();

		try {
			TransformerFactory.newInstance().newTransformer()
					.transform(new DOMSource(document1), new StreamResult(output1));
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Error transforming XML", ex);
			}
		}

		Map<String, String> requestProperties = new HashMap<String, String>();
		requestProperties.put("Content-type", "text/xml; charset=utf-8");
		StringBuffer xmlOut;
		try {
			String responseData = sendHTTPPost(authUrl, output1.toString(), true, requestProperties, "utf-8");
			xmlOut = new StringBuffer(responseData);
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Error sending request to authorize.net: ", ex);
			}
			return null;
		}

		try {
			results = parseProfilePageResponseXML(xmlOut.toString());
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Error parsing XML result", ex);
			}
		}

		return results;

	}

	private Map<String, String> parseProfilePageResponseXML(String xmlResponse) throws Exception {

		String xmlResultCode = null;
		String xmlMessageCode = null;
		String xmlMessageText = null;
		String token = null;
		Map<String, String> results = new HashMap<String, String>();

		Document doc = null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		// I can't believe we have to remove BOM here
		xmlResponse = xmlResponse.trim().replaceFirst("^([\\W]+)<", "<");
		InputSource is = new InputSource(new StringReader(xmlResponse));
		doc = builder.parse(is);
		NodeList messagesList = doc.getElementsByTagName("messages");
		int numMessages = messagesList.getLength();
		for (int i = 0; i < numMessages; i++) {
			Element messages = (Element) messagesList.item(i);
			Node n = messages.getFirstChild();
			while (n != null) {
				if (n.getNodeName().equals("resultCode")) {
					xmlResultCode = n.getFirstChild().getNodeValue();
				}
				if (n.getNodeName().equals("message")) {
					Node message = n.getFirstChild();
					while (message != null) {
						if (message.getNodeName().equals("code")) {
							xmlMessageCode = message.getFirstChild().getNodeValue();
						}
						if (message.getNodeName().equals("text")) {
							xmlMessageText = message.getFirstChild().getNodeValue();
						}
						message = message.getNextSibling();
					}
				}
				n = n.getNextSibling();
			}
		}

		NodeList tokenList = doc.getElementsByTagName("token");
		int numTokenList = tokenList.getLength();
		for (int i = 0; i < numTokenList; i++) {
			Element tokens = (Element) tokenList.item(i);
			Node n = tokens.getFirstChild();
			token = n.getNodeValue();
		}

		results.put("xmlResultCode", xmlResultCode);
		results.put("xmlMessageCode", xmlMessageCode);
		results.put("xmlMessageText", xmlMessageText);
		results.put("token", token);
		return results;

	}

	public AuthorizeNetCIMResultCreateProfile createCustomerProfile(Customer customer, Collection<Order> orders,
			Order liveOrder) throws Exception {
		boolean error = false;
		AuthorizeNetCIMResultCreateProfile authorizeNetCIMResult = new AuthorizeNetCIMResultCreateProfile();
		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);

		CustomerProfileType customerProfile = new CustomerProfileType();
		customerProfile.setMerchantCustomerId("c-" + Integer.toString(customer.getCustomerID()));
		customerProfile.setEmail(StringUtils.stripToEmpty(customer.getEmail1()));
		customerProfile.setDescription(StringUtils.stripToEmpty(customer.getUserName()));

		CustomerGatewayDAO dao = (CustomerGatewayDAO) getBusinessObjectFactory().getDaoFactory().createDAO(
				"customerGatewayDAOImplementer");

		// Add a the customer address info as the first shipping address which
		// would be used in any form as a default if payment does not have a
		// billing address.
		CustomerAddressType shipTo = null;
		try {
			if (customer.getFirstAddress() != null
					&& StringUtils.isNotBlank(customer.getFirstAddress().getPostalCode())) {
				shipTo = buildCustomerAddressType(PropertyUtils.describe(customer.getFirstAddress()), false);
			} else if (liveOrder != null && liveOrder.firstDelivery() != null
					&& StringUtils.isNotBlank(liveOrder.firstDelivery().getPostalCode())) {
				shipTo = buildCustomerAddressType(PropertyUtils.describe(liveOrder.firstDelivery()), false);
			} else {
				// Just try the customer
				shipTo = buildCustomerAddressType(PropertyUtils.describe(customer), false);
			}

		} catch (AuthorizeNetCIMParamException aex) {
			shipTo = null;
			if (log.isInfoEnabled()) {
				log.info("CIM Ship To Error: " + aex.getMessage());
				log.info("Allowing to continue adding profile without ship address.");
			}
		}

		if (shipTo != null) {
			ArrayOfCustomerAddressType shipList = new ArrayOfCustomerAddressType();
			shipList.getCustomerAddressType().add(shipTo);
			customerProfile.setShipToList(shipList);
		}

		// Used to avoid duplicate CC data which causes an auth.net error and
		// the entire transaction dies. This will end up using the latest
		// payment on the latest order for a single credit card. Each card
		// should have the latest data as long as the creditCardDate is entered.
		Map<String, Map> paymentData = new HashMap<String, Map>();

		if (orders != null && orders.size() > 0) {
			for (Order order : orders) {
				if (order.getCompleted() != null) {
					Map billToParams = PropertyUtils.describe(order);
					Collection<Payment> payments = order.getPayments();
					if (payments != null && payments.size() > 0) {
						for (Payment payment : payments) {
							if (payment.getCreditCardNumber() != null) {
								payment.setDecryptedNumber(dao.getDAOUtils().getDecrypted(
										payment.getCreditCardNumber(),
										(String) getSettings().getValue("paymentEncryptionType"),
										getAppSettings().getProperty("twoWayKeyFile")));
							}
							if (StringUtils.isBlank(payment.getDecryptedNumber())
									|| StringUtils.isBlank(payment.getCreditCardName())) {
								continue;
							}
							String key = payment.getDecryptedNumber();
							Map dataParams = PropertyUtils.describe(payment);
							dataParams.putAll(billToParams);
							paymentData.put(key, dataParams);
						}
					}
				}
			}
		}
		if (paymentData != null && !paymentData.isEmpty()) {
			ArrayOfCustomerPaymentProfileType paymentList = new ArrayOfCustomerPaymentProfileType();
			Iterator<Map.Entry<String, Map>> i = paymentData.entrySet().iterator();
			while (i.hasNext()) {
				Map.Entry<String, Map> entry = i.next();
				Map data = (Map) entry.getValue();
				CustomerPaymentProfileType paymentProfileType = null;
				try {
					paymentProfileType = buildCustomerPaymentProfileType(data, false);
				} catch (AuthorizeNetCIMParamException aex) {
					paymentProfileType = null;
					if (log.isInfoEnabled()) {
						log.info("CIM Payment Profile Error: " + aex.getMessage());
						log.info("Skipping current payment.");
					}
				}
				if (paymentProfileType != null) {
					paymentList.getCustomerPaymentProfileType().add(paymentProfileType);
				}
			}
			customerProfile.setPaymentProfiles(paymentList);
		}

		log.info("Sending create customer profile request: " + customerProfile.getMerchantCustomerId());
		CreateCustomerProfileResponseType response = soap.createCustomerProfile(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), customerProfile,
				ValidationModeEnum.fromValue(VALIDATION_MODE));
		log.info("Received create customer profile response with result code: " + response.getResultCode());

		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
			authorizeNetCIMResult.setProfileID(response.getCustomerProfileId());

			if (response.getCustomerPaymentProfileIdList() != null) {
				for (long paymentProfileId : response.getCustomerPaymentProfileIdList().getLong()) {
					authorizeNetCIMResult.getPaymentProfileIDs().add(paymentProfileId);
				}
			}

			if (response.getCustomerShippingAddressIdList() != null) {
				for (long shippingAddressId : response.getCustomerShippingAddressIdList().getLong()) {
					authorizeNetCIMResult.getShippingAddressIDs().add(shippingAddressId);
				}
			}
		}

		return authorizeNetCIMResult;

	}

	/**
	 * Creates an anonymous CIM account from the information in the current
	 * order. The customer will not be able to use this again, but the admin may
	 * use it.
	 * 
	 * @param order
	 * @throws Exception
	 */
	public AuthorizeNetCIMResultCreateProfile createCustomerProfile(Order order) throws Exception {
		boolean error = false;
		AuthorizeNetCIMResultCreateProfile authorizeNetCIMResult = new AuthorizeNetCIMResultCreateProfile();
		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);

		CustomerProfileType customerProfile = new CustomerProfileType();
		customerProfile.setMerchantCustomerId("o-" + Integer.toString(order.getOrderID()));
		customerProfile.setEmail(StringUtils.stripToEmpty(order.getEmail1()));
		customerProfile.setDescription(StringUtils.stripToEmpty(order.getFirstName()) + " "
				+ StringUtils.stripToEmpty(order.getLastName()));

		// Add a the customer address info as the first shipping address which
		// would be used in any form as a default if payment does not have a
		// billing address.
		CustomerAddressType shipTo = null;
		try {
			if (order.firstDelivery() != null && StringUtils.isNotBlank(order.firstDelivery().getPostalCode())) {
				shipTo = buildCustomerAddressType(PropertyUtils.describe(order.firstDelivery()), false);
			} else {
				// Try try the billing
				shipTo = buildCustomerAddressType(PropertyUtils.describe(order), false);
			}

		} catch (AuthorizeNetCIMParamException aex) {
			shipTo = null;
			if (log.isInfoEnabled()) {
				log.info("CIM Ship To Error: " + aex.getMessage());
				log.info("Allowing to continue adding profile without ship address.");
			}
		}

		if (shipTo != null) {
			ArrayOfCustomerAddressType shipList = new ArrayOfCustomerAddressType();
			shipList.getCustomerAddressType().add(shipTo);
			customerProfile.setShipToList(shipList);
		}

		CreateCustomerProfileResponseType response = soap.createCustomerProfile(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), customerProfile,
				ValidationModeEnum.fromValue(VALIDATION_MODE));

		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
			authorizeNetCIMResult.setProfileID(response.getCustomerProfileId());

			if (response.getCustomerPaymentProfileIdList() != null) {
				for (long paymentProfileId : response.getCustomerPaymentProfileIdList().getLong()) {
					authorizeNetCIMResult.getPaymentProfileIDs().add(paymentProfileId);
				}
			}

			if (response.getCustomerShippingAddressIdList() != null) {
				for (long shippingAddressId : response.getCustomerShippingAddressIdList().getLong()) {
					authorizeNetCIMResult.getShippingAddressIDs().add(shippingAddressId);
				}
			}
		}

		return authorizeNetCIMResult;

	}

	public AuthorizeNetCIMResult updateCustomerPaymentProfile(long cimProfileID,
			CustomerPaymentProfileMaskedType paymentProfile) throws Exception {

		boolean error = false;
		// Have to create a new ex type as CIM wont take update from Masked
		// type.
		CustomerPaymentProfileExType paymentProfileExType = new CustomerPaymentProfileExType();
		paymentProfileExType.setBillTo(paymentProfile.getBillTo());
		paymentProfileExType.setCustomerPaymentProfileId(paymentProfile.getCustomerPaymentProfileId());
		paymentProfileExType.setCustomerType(paymentProfile.getCustomerType());
		if (paymentProfile.getDriversLicense() != null) {
			DriversLicenseType dlt = new DriversLicenseType();
			dlt.setDateOfBirth(paymentProfile.getDriversLicense().getDateOfBirth());
			dlt.setNumber(paymentProfile.getDriversLicense().getNumber());
			dlt.setState(paymentProfile.getDriversLicense().getState());
		}
		if (paymentProfile.getPayment() != null) {
			PaymentType pt = new PaymentType();
			if (paymentProfile.getPayment().getBankAccount() != null) {
				BankAccountType bat = new BankAccountType();
				bat.setAccountNumber(paymentProfile.getPayment().getBankAccount().getAccountNumber());
				bat.setAccountType(paymentProfile.getPayment().getBankAccount().getAccountType());
				bat.setBankName(paymentProfile.getPayment().getBankAccount().getBankName());
				bat.setEcheckType(paymentProfile.getPayment().getBankAccount().getEcheckType());
				bat.setNameOnAccount(paymentProfile.getPayment().getBankAccount().getNameOnAccount());
				bat.setRoutingNumber(paymentProfile.getPayment().getBankAccount().getRoutingNumber());
				pt.setBankAccount(bat);
			}
			if (paymentProfile.getPayment().getCreditCard() != null) {
				CreditCardType cct = new CreditCardType();
				cct.setCardNumber(paymentProfile.getPayment().getCreditCard().getCardNumber());
				cct.setExpirationDate(paymentProfile.getPayment().getCreditCard().getExpirationDate());
				pt.setCreditCard(cct);
			}
			paymentProfileExType.setPayment(pt);
		}
		paymentProfileExType.setTaxId(paymentProfile.getTaxId());

		AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();
		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);
		// We assume someone has updated the paymentProfile bean with info at
		// this point, we are just sending it along.

		UpdateCustomerPaymentProfileResponseType response = soap.updateCustomerPaymentProfile(AuthorizeNetCIMService
				.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), cimProfileID, paymentProfileExType,
				ValidationModeEnum.fromValue(VALIDATION_MODE));

		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
			authorizeNetCIMResult.setError(true);
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
			authorizeNetCIMResult.setPaymentProfileID(paymentProfile.getCustomerPaymentProfileId());
			authorizeNetCIMResult.setValidationDirectResponse(response.getValidationDirectResponse());
		}

		return authorizeNetCIMResult;


	}

	public AuthorizeNetCIMResult updateCustomerPaymentProfile(long cimProfileID, long paymentProfileID,
			Payment payment, Order order) throws Exception {
		boolean error = false;

		AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();
		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);

		CustomerGatewayDAO dao = (CustomerGatewayDAO) getBusinessObjectFactory().getDaoFactory().createDAO(
				"customerGatewayDAOImplementer");
		Map billToParams = PropertyUtils.describe(order);
		if (payment.getCreditCardNumber() != null) {
			payment.setDecryptedNumber(dao.getDAOUtils().getDecrypted(payment.getCreditCardNumber(),
					(String) getSettings().getValue("paymentEncryptionType"),
					getAppSettings().getProperty("twoWayKeyFile")));
		}

		if (StringUtils.isBlank(payment.getDecryptedNumber()) || StringUtils.isBlank(payment.getCreditCardDate())
				|| StringUtils.isBlank(payment.getCreditCardName())) {
			// Blank credit card
			authorizeNetCIMResult.setError(true);
			Map<String, String> e = new HashMap<String, String>();
			if (StringUtils.isBlank(payment.getDecryptedNumber())) {
				e.put("code", "InternalError_DecryptedNumber");
			} else if (StringUtils.isBlank(payment.getCreditCardDate())) {
				e.put("code", "InternalError_CreditCardDate");
			} else if (StringUtils.isBlank(payment.getCreditCardName())) {
				e.put("code", "InternalError_CreditCardName");
			} else {
				e.put("code", "InternalError");
			}
			e.put("text", "Unable to get credit card name, number, or expiration");
			authorizeNetCIMResult.getMessages().add(e);
			return authorizeNetCIMResult;
		}

		Map dataParams = PropertyUtils.describe(payment);
		dataParams.putAll(billToParams);
		CustomerPaymentProfileExType paymentProfileExType = null;

		try {
			paymentProfileExType = buildCustomerPaymentProfileType(dataParams, false);
		} catch (AuthorizeNetCIMParamException aex) {
			paymentProfileExType = null;
			if (log.isInfoEnabled()) {
				log.info("CIM Payment Profile Error: " + aex.getMessage());
			}
			Map<String, String> e = new HashMap<String, String>();
			e.put("code", "InternalError");
			e.put("text", aex.getMessage());
			authorizeNetCIMResult.setError(true);
			authorizeNetCIMResult.getMessages().add(e);
			return authorizeNetCIMResult;
		}
		if (paymentProfileExType == null) {
			// No profile created?
			authorizeNetCIMResult.setError(true);
			Map<String, String> e = new HashMap<String, String>();
			e.put("code", "InternalError");
			e.put("text", "Unable to build payment profile type to send.");
			authorizeNetCIMResult.getMessages().add(e);
			return authorizeNetCIMResult;
		}

		paymentProfileExType.setCustomerPaymentProfileId(paymentProfileID);

		UpdateCustomerPaymentProfileResponseType response = soap.updateCustomerPaymentProfile(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), cimProfileID,
				paymentProfileExType, ValidationModeEnum.fromValue(VALIDATION_MODE));

		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
			authorizeNetCIMResult.setError(true);
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
			authorizeNetCIMResult.setPaymentProfileID(paymentProfileID);
			authorizeNetCIMResult.setValidationDirectResponse(response.getValidationDirectResponse());
		}

		return authorizeNetCIMResult;

	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public AuthorizeNetCIMResult createCustomerPaymentProfile(long cimProfileID, Payment payment, Order order)
			throws Exception {

		Map billToParams = PropertyUtils.describe(order);
		Map paymentParams = PropertyUtils.describe(payment);
		paymentParams.putAll(billToParams);
		return createCustomerPaymentProfile(cimProfileID, paymentParams, false);
	}

	@SuppressWarnings("unchecked")
	public AuthorizeNetCIMResult createCustomerPaymentProfile(long cimProfileID, Map paymentParams, boolean billing)
			throws Exception {
		boolean error = false;

		AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();
		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);

		CustomerGatewayDAO dao = (CustomerGatewayDAO) getBusinessObjectFactory().getDaoFactory().createDAO(
				"customerGatewayDAOImplementer");

		String creditCardNumber = (String) paymentParams.get("creditCardNumber");
		if (StringUtils.isNotBlank(creditCardNumber)) {
			paymentParams.put(
					"decryptedNumber",
					dao.getDAOUtils().getDecrypted(creditCardNumber,
							(String) getSettings().getValue("paymentEncryptionType"),
							getAppSettings().getProperty("twoWayKeyFile")));
		}
		String decryptedNumber = (String) paymentParams.get("decryptedNumber");
		String creditCardDate = (String) paymentParams.get("creditCardDate");

		if (StringUtils.isBlank(decryptedNumber) || StringUtils.isBlank(creditCardDate)) {
			// Blank credit card
			authorizeNetCIMResult.setError(true);
			Map<String, String> e = new HashMap<String, String>();
			if (StringUtils.isBlank(decryptedNumber)) {
				e.put("code", "InternalError_DecryptedNumber");
			} else if (StringUtils.isBlank(creditCardDate)) {
				e.put("code", "InternalError_CreditCardDate");
			} else {
				e.put("code", "InternalError");
			}
			e.put("text", "Unable to get credit, number, or expiration");
			authorizeNetCIMResult.getMessages().add(e);
			return authorizeNetCIMResult;
		}

		CustomerPaymentProfileType paymentProfileType = null;

		try {
			paymentProfileType = buildCustomerPaymentProfileType(paymentParams, billing);
		} catch (AuthorizeNetCIMParamException aex) {
			paymentProfileType = null;
			if (log.isInfoEnabled()) {
				log.info("CIM Payment Profile Error: " + aex.getMessage());
			}
			Map<String, String> e = new HashMap<String, String>();
			authorizeNetCIMResult.setError(true);
			e.put("code", "InternalError");
			e.put("text", aex.getMessage());
			authorizeNetCIMResult.getMessages().add(e);
			return authorizeNetCIMResult;
		}
		if (paymentProfileType == null) {
			// No profile created?
			authorizeNetCIMResult.setError(true);
			Map<String, String> e = new HashMap<String, String>();
			e.put("code", "InternalError");
			e.put("text", "Unable to build payment profile type to send.");
			authorizeNetCIMResult.getMessages().add(e);
			return authorizeNetCIMResult;
		}

		CreateCustomerPaymentProfileResponseType response = soap.createCustomerPaymentProfile(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), cimProfileID,
				paymentProfileType, ValidationModeEnum.fromValue(VALIDATION_MODE));

		if (response == null || !MessageTypeEnum.OK.equals(response.getResultCode())) {
			error = true;
			authorizeNetCIMResult.setError(true);
		}

		if (response != null) {
			authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);
			authorizeNetCIMResult.setPaymentProfileID(response.getCustomerPaymentProfileId());
			authorizeNetCIMResult.setValidationDirectResponse(response.getValidationDirectResponse());
		}

		return authorizeNetCIMResult;

	}

	public AuthorizeNetCIMResult deleteCustomerPaymentProfile(long profileID, long paymentProfileID) {
		boolean error = false;
		AuthorizeNetCIMResult authorizeNetCIMResult = new AuthorizeNetCIMResult();
		ServiceSoap soap = AuthorizeNetCIMService.getServiceSoap(WSDL_URL, SERVICE_URL, false);
		DeleteCustomerPaymentProfileResponseType response = soap.deleteCustomerPaymentProfile(
				AuthorizeNetCIMService.getMerchantAuthentication(MERCHANT_NAME, TRANSACTION_KEY), profileID,
				paymentProfileID);
		if (!response.getResultCode().equals(MessageTypeEnum.OK)) {
			error = true;
		}
		authorizeNetCIMResult.initialize(response.getMessages().getMessagesTypeMessage(), error);

		return authorizeNetCIMResult;
	}
	

	public List<Object[]> loadCustomersWithNoCIMProfileByDate(
			String createdStart, String createdEnd) throws Exception {
		// Query for the orders
		CustomerGatewayDAO dao = (CustomerGatewayDAO) getBusinessObjectFactory()
				.getDaoFactory().createDAO("customerGatewayDAOImplementer");
		return dao
				.loadCustomersWithNoCIMProfileByDate(createdStart, createdEnd);
	}

	public List<Object[]> loadPaymentsWithNoProfileByDate(String createdStart,
			String createdEnd) throws Exception {
		// Query for the orders
		PaymentGatewayDAO dao = (PaymentGatewayDAO) getBusinessObjectFactory()
				.getDaoFactory().createDAO("paymentGatewayDAOImplementer");
		return dao.loadPaymentsWithNoProfileByDate(createdStart, createdEnd);
	}

	public Collection<Customer> loadCustomersWithNoCIMProfileByDate(String cutoffDate) throws Exception {
		// Query for the orders
		CustomerGatewayDAO dao = (CustomerGatewayDAO) getBusinessObjectFactory().getDaoFactory().createDAO(
				"customerGatewayDAOImplementer");
		return dao.loadCustomersWithNoCIMProfileByDate(cutoffDate);
	}

	public Collection<Payment> loadPaymentsWithNoCIMProfileByDate(String cutoffDate) throws Exception {
		// Query for the orders
		PaymentGatewayDAO dao = (PaymentGatewayDAO) getBusinessObjectFactory().getDaoFactory().createDAO(
				"paymentGatewayDAOImplementer");
		return dao.loadPaymentsWithNoCIMProfileByDate(cutoffDate);
	}

	public void initialize() {
		TEST_MODE = (String) getSettings().getValue("authorizeNetCIMTestRequest");
		if ("1".equals(TEST_MODE)) {
			WSDL_URL = (String) getSettings().getValue("authorizeNetCIMTestWSDL");
		} else {
			WSDL_URL = (String) getSettings().getValue("authorizeNetCIMWSDL");
		}

		SERVICE_URL = (String) getSettings().getValue("authorizeNetCIMServiceURL");
		VALIDATION_MODE = (String) getSettings().getValue("authorizeNetCIMValidationMode");
		MERCHANT_NAME = (String) getSettings().getValue("authorizeNetCIMLoginID");
		TRANSACTION_KEY = (String) getSettings().getValue("authorizeNetCIMTxKey");

	}

	@SuppressWarnings("unchecked")
	protected CustomerPaymentProfileExType buildCustomerPaymentProfileType(Map parameters, boolean billing)
			throws Exception {

		// Card Data
		String decryptedNumber = StringUtils.stripToEmpty((String) parameters.get("decryptedNumber"));
		String creditCardDate = StringUtils.stripToEmpty((String) parameters.get("creditCardDate"));
		String securityCode = StringUtils.stripToEmpty((String) parameters.get("securityCode"));

		if (StringUtils.isBlank(decryptedNumber) || StringUtils.isBlank(creditCardDate)) {
			throw new AuthorizeNetCIMParamException("Invalid credit card number or credit card date");
		}

		CustomerPaymentProfileExType paymentProfile = new CustomerPaymentProfileExType();
		PaymentType p = new PaymentType();
		CreditCardType creditCard = new CreditCardType();
		creditCard.setCardNumber(decryptedNumber);
		if (!ValidationModeEnum.NONE.equals(VALIDATION_MODE)) {
			creditCard.setCardCode(securityCode);
		}
		String ccMonth = "";
		String ccYear = "";
		if (creditCardDate.length() > 5) {
			ccMonth = creditCardDate.substring(4);
			ccYear = creditCardDate.substring(0, 4);
		}
		creditCard.setExpirationDate(ccYear + "-" + ccMonth);
		p.setCreditCard(creditCard);
		paymentProfile.setPayment(p);

		CustomerAddressType billTo = null;
		try {
			billTo = buildCustomerAddressType(parameters, billing);
		} catch (AuthorizeNetCIMParamException aex) {
			billTo = null;
			if (log.isInfoEnabled()) {
				log.info("CIM Bill To Error: " + aex.getMessage());
			}
			throw (aex);
		}

		paymentProfile.setBillTo(billTo);

		return paymentProfile;
	}

	public CustomerAddressType buildCustomerAddressType(Map parameters, boolean billing) throws Exception {
		CustomerAddressType address = new CustomerAddressType();
		
		String firstName = null;
		String lastName = null;
		String organization = null;
		String address1 = null;
		String address2 = null;
		String city = null;
		String state = null;
		String postalCode = null;
		String country = null;
		String phone1 = null;
		String phone2 = null;
		
		if (billing){
			firstName = (String) parameters.get("billingFirstName");
			lastName = (String) parameters.get("billingLastName");
			organization = (String) parameters.get("billingOrganization");
			address1 = StringUtils.stripToEmpty((String) parameters.get("billingAddress1"));
			address2 = StringUtils.stripToEmpty((String) parameters.get("billingAddress2"));
			city = (String) parameters.get("billingCity");
			state = (String) parameters.get("billingState");
			postalCode = (String) parameters.get("billingPostalCode");
			country = (String) parameters.get("billingCountry");
			phone1 = (String) parameters.get("billingPhone1");
			phone2 = (String) parameters.get("billingPhone2");
		}else{
			firstName = (String) parameters.get("firstName");
			lastName = (String) parameters.get("lastName");
			organization = (String) parameters.get("organization");
			address1 = StringUtils.stripToEmpty((String) parameters.get("address1"));
			address2 = StringUtils.stripToEmpty((String) parameters.get("address2"));
			city = (String) parameters.get("city");
			state = (String) parameters.get("state");
			postalCode = (String) parameters.get("postalCode");
			country = (String) parameters.get("country");
			phone1 = (String) parameters.get("phone1");
			phone2 = (String) parameters.get("phone2");
		}
		
		address.setFirstName(fixString(firstName, 50));
		address.setLastName(fixString(lastName, 50));
		address.setCompany(fixString(organization, 50));
		String newAddress;
		if (address1.length() > 0 && address2.length() > 0) {
			newAddress = address1 + " " + address2;
		} else {
			newAddress = address1 + address2;
		}
		address.setAddress(fixString(newAddress, 60));
		address.setCity(fixString(city, 40));
		address.setState(fixString(state, 40));
		address.setZip(fixString(postalCode, 20));
		address.setCountry(fixString(country, 60));
		address.setPhoneNumber(fixString(phone1, 25, true));
		address.setFaxNumber(fixString(phone2, 25, true));

		if (address.getFirstName().length() > 0 || address.getLastName().length() > 0
				|| address.getCompany().length() > 0 || address.getAddress().length() > 0
				|| address.getCity().length() > 0 || address.getState().length() > 0 || address.getZip().length() > 0
				|| address.getCountry().length() > 0 || address.getPhoneNumber().length() > 0
				|| address.getFaxNumber().length() > 0) {
			return address;
		} else {
			// CIM will error if we give them a blank address, so return null.
			// The caller should not add an address to the profile if this is
			// null.
			throw new AuthorizeNetCIMParamException("One or more address values must be valid.");
		}

	}

	protected String fixString(String val, int maxChars) {
		return fixString(val, maxChars, false);
	}

	protected String fixString(String val, int maxChars, boolean numbersOnly) {
		if (val != null) {
			if (numbersOnly) {
				val = val.replaceAll("\\D", "");
			} else {
				val = val.replaceAll("&", "and");
				val = val.replaceAll("\\<", "");
				val = val.replaceAll("\\>", "");
			}

			if (val.length() > maxChars) {
				val = val.substring(0, maxChars - 1);
			}
		} else {
			val = "";
		}
		return val;
	}

	public Map processSuccessResponse(Map parameters, AuthorizeNetCIMResult authorizeNetCIMResult,
                                         boolean priorAuthCapture) throws Exception {
		Map result = new HashMap();

		String directResponse = authorizeNetCIMResult.getDirectResponse();
		String[] responseArray = directResponse.split("\\|");
		List response = java.util.Arrays.asList(responseArray);

		if (log.isDebugEnabled())
			printResponseArray(response);

		result.put("resultCode", "0");
		result.put("resultMessage", response.get(AuthorizeNetAIMConstants.RESPONSE_REASON_TEXT));

		Payment payment =  getBusinessObjectFactory().createObject(Payment.class);
		BeanUtils.copyProperties(payment, parameters);
		payment.setStatus("Received");
		try {
			String amountS = (String) response.get(AuthorizeNetAIMConstants.RESPONSE_AMOUNT);
			Double amountD = new Double(amountS);
			payment.setAmount(amountD);
		} catch (Exception e) {
			if (log.isWarnEnabled()) {
				log.warn("Error parsing payment amount: ", e);
			}
		}
		payment.setCreated(formatDateTime(new java.util.Date()));
		payment.setProcessorClassName(this.getClass().getName());
		payment.setCreditCardNumberDisplay((String) response.get(AuthorizeNetAIMConstants.RESPONSE_ACCOUNT_NUMBER));
		payment.setCreditCardType((String) response.get(AuthorizeNetAIMConstants.RESPONSE_CARD_TYPE));
		payment.setValueFieldLabels("TRANSACTION_TYPE,AMOUNT,CARD_CODE,RESPONSE_CODE,AVS_CODE,AUTHCODE,TRANSACTION_ID,REASON_CODE,REASON_TEXT");
		payment.setValue1((String) response.get(AuthorizeNetAIMConstants.RESPONSE_TRANSACTION_TYPE));
		payment.setValue2((String) response.get(AuthorizeNetAIMConstants.RESPONSE_AMOUNT));
		if (response.size() > AuthorizeNetAIMConstants.RESPONSE_CVV2_CODE) {
			payment.setValue3((String) response.get(AuthorizeNetAIMConstants.RESPONSE_CVV2_CODE));
		}
		payment.setValue4((String) response.get(AuthorizeNetAIMConstants.RESPONSE_CODE));
		payment.setValue5((String) response.get(AuthorizeNetAIMConstants.RESPONSE_AVS_CODE));
		payment.setValue6((String) response.get(AuthorizeNetAIMConstants.RESPONSE_AUTH_CODE));
		payment.setValue7((String) response.get(AuthorizeNetAIMConstants.RESPONSE_TRANSACTION_ID));
		payment.setValue8((String) response.get(AuthorizeNetAIMConstants.RESPONSE_REASON_CODE));
		payment.setValue9((String) response.get(AuthorizeNetAIMConstants.RESPONSE_REASON_TEXT));
		if (authorizeNetCIMResult.getPaymentProfileID() > 0) {
			payment.setPaymentProfileID(Long.toString(authorizeNetCIMResult.getPaymentProfileID()));
		}
        if(priorAuthCapture) {
            // We do not have a user since this is triggered from the events system
            Order order = (Order) parameters.get("order");
            payment.setOrder(order);
        }
        else {
		payment.setOrder(getUser().getOrder());
		if (getUser().getOrder().getPayments() == null)
			getUser().getOrder().setPayments(new LinkedHashSet());
		getUser().getOrder().getPayments().add(payment);
        }
		PaymentDAO paymentDAO =  getDaoFactory().createDAO(PaymentDAO.class);
		paymentDAO.setPayment(payment);
		paymentDAO.insertPayment(false);
		result.put("newPayment", payment);

		return result;
	}

	private void backFillBillingAddress(AuthorizeNetCIMAddress address, Order order) throws Exception {

		order.setFirstName(address.getFirstName());
		order.setLastName(address.getLastName());
		order.setOrganization(address.getCompany());
		order.setAddress1(address.getAddress());
		order.setCity(address.getCity());
		order.setState(address.getState());
		order.setPostalCode(address.getZip());
		order.setCountry(address.getCountry());
		order.setPhone1(address.getPhoneNumber());

	}

	public Map<String, String> logProfileErrors(String internalErrorMessage, List<Map<String, String>> messages,
			Customer customer, Order order, String command) {
		Map<String, String> firstError = null;
		if (log.isInfoEnabled()) {
			if (customer != null && customer.getCustomerID() > 0) {
				log.info("CIM error " + command + " - customer: " + customer.getCustomerID());
			}
			if (order != null && order.getOrderID() > 0) {
				log.info("CIM error " + command + " - order: " + order.getOrderID());
			}
			log.info(StringUtils.stripToEmpty(internalErrorMessage));
		}
		if (messages != null) {
			for (Map<String, String> message : messages) {
				if (log.isInfoEnabled()) {
					log.info(StringUtils.stripToEmpty(message.get("code")) + ": "
							+ StringUtils.stripToEmpty(message.get("text")));
				}
				if (firstError == null) {
					firstError = new HashMap<String, String>();
					firstError.put("code", (StringUtils.stripToEmpty(message.get("code"))));
					firstError.put("text", (StringUtils.stripToEmpty(message.get("text"))));
				}
			}
		}
		return firstError;

	}

	private void buildTransactionLineItems(List<LineItemType> transItems, Collection<OrderItem> orderItems) {

		if (orderItems != null) {
			int added = 0;
			for (OrderItem oi : orderItems) {
				LineItemType transItem = new LineItemType();
				transItem.setItemId(fixString(oi.getCode(), 31));
				transItem.setName(fixString(oi.getName(), 31));
				transItem.setDescription(fixString(oi.getShortDescription(), 255));
				transItem.setQuantity(new BigDecimal(String.valueOf(oi.getQuantity())));
				transItem.setTaxable(oi.getIsTaxed());
				Double unitPrice = oi.getTotal().doubleValue() / oi.getQuantity();
				unitPrice = utils().roundForMoney(unitPrice);
				if (unitPrice != null) {
					transItem.setUnitPrice(new BigDecimal(unitPrice.toString()));
				}
				transItems.add(transItem);
				added++;
				if (added > 29) {
					break;
				}
			}
		}
	}

	protected void printResponseArray(List l) {
		log.debug("Printing response (" + l.size() + " elements)");
		for (int i = 0; i < l.size(); i++)
			log.debug(i + " " + l.get(i));
	}

	public Map<String, String> validatePaymentMethod(Map<String, String> parameters) throws Exception {

		Map<String, String> result = new HashMap<String, String>();
		Order order = null;
		Customer customer = null;
		long useProfileID = 0;
		long cimPaymentProfileID = 0;
		String merchantCustomerID = null;
		String cimPaymentProfileIDStr = (String) parameters.get("cimPaymentProfileID");
		String cimPaymentProfileType = (String) parameters.get("cimPaymentProfileType");

		try {
			cimPaymentProfileID = Long.parseLong(cimPaymentProfileIDStr);
		} catch (Exception ex) {
			cimPaymentProfileID = 0;
		}

		if (cimPaymentProfileID < 1) {
			result.put("resultCode", "90");
			result.put("resultMessage", null);
			return result;
		}

		if (!"order".equalsIgnoreCase(cimPaymentProfileType) && !"customer".equalsIgnoreCase(cimPaymentProfileType)) {
			result.put("resultCode", "90");
			result.put("resultMessage", null);
			return result;
		}

		if (getUser() != null && getUser().getOrder() != null) {
			order = getUser().getOrder();
		}
		if (getUser() != null && getUser().getCustomer() != null) {
			customer = getUser().getCustomer();
		}
		if (order == null) {
			return null;
		}

		if ("customer".equalsIgnoreCase(cimPaymentProfileType)) {
			if (StringUtils.isNotBlank(customer.getCimProfileID())) {
				useProfileID = Long.parseLong(customer.getCimProfileID());
				merchantCustomerID = "c-" + customer.getCustomerID();
			}
		} else {
			if (StringUtils.isNotBlank(order.getPaymentProfileID())) {
				useProfileID = Long.parseLong(order.getPaymentProfileID());
				merchantCustomerID = "o-" + order.getOrderID();
			}
		}

		AuthorizeNetCIMResultGetProfile getProfileResult = getCustomerProfile(useProfileID);
		AuthorizeNetCIMPaymentProfileMin matchingPaymentProfile = null;

		if (getProfileResult.getError()) {
			logProfileErrors("Error getting profile inforamtion.", getProfileResult.getMessages(), customer, order,
					"getCustomerProfile");
			result.put("resultCode", "92");
			result.put("resultMessage", null);
			return result;

		} else {
			if (getProfileResult.getProfileID() > 0 && getProfileResult.getProfileID() == useProfileID
					&& merchantCustomerID.equals(getProfileResult.getMerchantCustomerID())) {

				if (getProfileResult.getPaymentProfiles() != null && getProfileResult.getPaymentProfiles().size() > 0) {
					for (AuthorizeNetCIMPaymentProfileMin profile : getProfileResult.getPaymentProfiles()) {
						if (profile.getCustomerPaymentProfileId() > 0
								&& profile.getCustomerPaymentProfileId() == cimPaymentProfileID) {
							matchingPaymentProfile = profile;
							break;
						}
					}
				}
			} else {
				// Not a good thing here
				logProfileErrors(
						"Returned profileID, merchantCustomerID does not match the sent profileID, merchantCustomerID",
						getProfileResult.getMessages(), customer, order, "getCustomerProfile");
				result.put("resultCode", "93");
				result.put("resultMessage", null);
				return result;
			}
		}

		if (matchingPaymentProfile == null) {
			logProfileErrors("Cant find matching profile for chosen profile.", getProfileResult.getMessages(),
					customer, order, "getCustomerProfile");
			result.put("resultCode", "93");
			result.put("resultMessage", null);
			return result;
		}

		String authorizeNetCIMFillBillingAddress = (String) getSettings().getValue("authorizeNetCIMFillBillingAddress");

		if ("1".equals(authorizeNetCIMFillBillingAddress) && getUser().getOrder() != null) {
			try {
				backFillBillingAddress(matchingPaymentProfile.getAddress(), getUser().getOrder());
			} catch (Exception ex) {
				if (log.isErrorEnabled()) {
					log.error("Exception back filling CIM billing address, allowing to continue: " + ex);
				}
			}
		}

		result.put("creditCardNumberDisplay", matchingPaymentProfile.getCardNumber());
		result.put("resultCode", "0");
		result.put("resultMessage", "");
		return result;

	}

	public void copyPaymentPaymentProfileToAddressForm(AuthorizeNetCIMPaymentProfileMin payment, BillingAddressForm form) {
		if (payment.getAddress() != null) {
			form.setBillingAddress1(payment.getAddress().getAddress());
			form.setBillingCity(payment.getAddress().getCity());
			form.setBillingOrganization(payment.getAddress().getCompany());
			form.setBillingCountry(payment.getAddress().getCountry());
			form.setBillingFirstName(payment.getAddress().getFirstName());
			form.setBillingLastName(payment.getAddress().getLastName());
			form.setBillingPhone1(payment.getAddress().getPhoneNumber());
			form.setBillingState(payment.getAddress().getState());
			form.setBillingPostalCode(payment.getAddress().getZip());
		}
	}

	public long findValidCimProfileID(Customer customer, Order order) {
		long cimProfileID = 0;
		if (customer != null && customer.getCustomerID() > 0 && StringUtils.isNotBlank(customer.getCimProfileID())) {
			// Valid Customer
			try {
				cimProfileID = Long.parseLong(customer.getCimProfileID());
			} catch (Exception ex) {
				cimProfileID = 0;
			}
			if (cimProfileID > 0) {
				if (log.isInfoEnabled()) {
					log.info("Found cim profile id for customer");
				}
			} else if (cimProfileID < 0) {
				if (log.isInfoEnabled()) {
					log.info("CIM Profiles are disabled for the customer for some reason.");
				}
			} else {
				if (log.isInfoEnabled()) {
					log.info("Cound not find a valid cimProfile ID for customer record.");
				}
			}
		}
		// No cimProfileID? Try order
		if (cimProfileID == 0) {
			if (log.isInfoEnabled()) {
				log.info("Trying to find anonymous CIM profile id on the order.");
			}
			// No customer, try the order record
			if (order != null && StringUtils.isNotBlank(order.getPaymentProfileID())) {
				try {
					cimProfileID = Long.parseLong(order.getPaymentProfileID());
				} catch (Exception ex) {
					cimProfileID = 0;
				}
			}
		}
		return cimProfileID;

	}

	public AuthorizeNetCIMPaymentProfileMin findMatchingPaymentProfile(Customer customer, Order order,
			String incomingPaymentProfile) throws Exception {

		long incomingCIMPaymentProfileID = 0;

		try {
			incomingCIMPaymentProfileID = Long.parseLong(incomingPaymentProfile);
		} catch (Exception ex) {
			incomingCIMPaymentProfileID = 0;
		}

		if (incomingCIMPaymentProfileID < 1) {
			if (log.isErrorEnabled()) {
				log.error("Invalid payment profile ID to look for: " + incomingPaymentProfile);
			}
			return null;
		}

		Collection<AuthorizeNetCIMPaymentProfileMin> paymentProfiles = findPaymentProfiles(customer, order);
		AuthorizeNetCIMPaymentProfileMin found = null;
		if (paymentProfiles != null && paymentProfiles.size() > 0) {
			for (AuthorizeNetCIMPaymentProfileMin paymentProfile : paymentProfiles) {
				if (paymentProfile.getCustomerPaymentProfileId() == incomingCIMPaymentProfileID) {
					found = paymentProfile;
					break;
				}
			}
		} else {
			if (log.isInfoEnabled()) {
				log.info("No payment profiles found.");
			}
			return null;
		}
		if (found != null) {
			return found;
		} else {
			if (log.isInfoEnabled()) {
				log.info("Profile found, but could not find the matching payment profile id.");
			}
			return null;
		}
	}

	public Collection<AuthorizeNetCIMPaymentProfileMin> findPaymentProfiles(Customer customer, Order order)
			throws Exception {

		long cimProfileID = findValidCimProfileID(customer, order);

		if (cimProfileID > 0) {
			AuthorizeNetCIMResultGetProfile getProfileResult = null;
			try {
				getProfileResult = getCustomerProfile(cimProfileID);
			} catch (javax.xml.ws.WebServiceException wse) {
				if (log.isErrorEnabled()) {
					log.error(wse);
				}
				return null;
			}
			if (getProfileResult.getError()) {
				if (log.isErrorEnabled()) {
					log.error("Error getting profile information: " + getProfileResult.getMessages());
				}
				return null;
			} else {
				if (getProfileResult.getProfileID() > 0 && getProfileResult.getProfileID() == cimProfileID) {
					if (getProfileResult.getPaymentProfiles() != null
							&& getProfileResult.getPaymentProfiles().size() > 0) {
						Collection<AuthorizeNetCIMPaymentProfileMin> paymentProfiles = getProfileResult
								.getPaymentProfiles();
						return paymentProfiles;
					} else {
						if (log.isInfoEnabled()) {
							log.info("Profile found, but user has no payment profiles yet.");
						}
						return null;
					}
				} else {
					if (log.isErrorEnabled()) {
						log.error("Returned profileID does not match sent profile id or profileID is 0.   Sent: "
								+ cimProfileID + " returned: " + getProfileResult.getProfileID());
					}
					return null;
				}
			}
		} else {
			if (log.isInfoEnabled()) {
				log.info("No valid cim profileID found.");
			}
			return null;
		}
	}

}
