package com.softslate.commerce.businessobjects.product;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods affecting products and other related
 * objects.
 * <p>
 * Instances implementing <code>ProductReviewProcessor</code> are usually
 * created in the Struts layer in response to a request from a user for product
 * review information. The <code>ProductReviewProcessor</code> object uses data
 * access objects to look up the information, and return it to the Struts layer.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ProductReviewProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "productReviewProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "productReviewProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.BasicProductReviewProcessor}.
 * 
 * @author Tony Zakula
 * 
 */

public interface ProductReviewProcessor extends BusinessProcessor {

	Map addProductReview(Map parameters) throws Exception;

	void deleteProductReview(Map parameters) throws Exception;

	Map editDeleteProductReviews(Map parameters) throws Exception;

	Map editProductReview(Map parameters) throws Exception;

	ProductReview loadProductReviewFromID(Map parameters) throws Exception;

	Map loadProductReviewsAndCount(Map parameters) throws Exception;

}
