package com.softslate.commerce.businessobjects.shipping;

import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a shipping method within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscShippingMethod</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "shippingMethodImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "shippingMethodImplementer" is
 * {@link com.softslate.commerce.businessobjects.shipping.ShippingMethodBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudShippingMethod")
public interface ShippingMethod extends BusinessObject {
	abstract int getShippingMethodID();

	abstract void setShippingMethodID(int shippingMethodID);

	abstract Double getBase();

	abstract void setBase(Double base);

	abstract String getCode();

	abstract void setCode(String code);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract String getMethodType();

	abstract void setMethodType(String methodType);

	abstract Double getMinimum();

	abstract void setMinimum(Double minimum);

	abstract String getName();

	abstract void setName(String name);

	abstract Collection getShippingRates();

	abstract void setShippingRates(Collection shippingRates);

	public Integer getNumberDaysToShip();

	public void setNumberDaysToShip(Integer numberDaysToShip);

	public Integer getMethodOrder();

	public void setMethodOrder(Integer methodOrder);
}
