package com.softslate.commerce.businessobjects.shipping;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface for business logic methods having to do with shipping rates.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting shipping rates.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ShippingRateProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "shippingRateProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "shippingRateProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.shipping.ShippingRateProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface ShippingRateProcessor extends BusinessObject {

	abstract Map loadShippingRatesAndCount(Map parameters) throws Exception;

	abstract Map addShippingRate(Map parameters) throws Exception;

	abstract ShippingRate loadShippingRateFromID(Map parameters)
			throws Exception;

	abstract Map editShippingRate(Map parameters) throws Exception;

	abstract Map editDeleteShippingRates(Map parameters) throws Exception;

	abstract void deleteShippingRate(Map parameters) throws Exception;
}
