package com.softslate.commerce.customer.customer;

import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMProcessor;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResult;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.order.CIMBillingAddressForm;

/**
 * 
 * @author Jason McWilliams
 * 
 */
public class AddCIMProfileAction extends BaseAction {
	static Log log = LogFactory.getLog(AddCIMProfileAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {

		if (log.isInfoEnabled())
			log.info("Starting action.");
		CIMBillingAddressForm baseForm = (CIMBillingAddressForm) form;
		synchronized (request.getSession()) {
			Properties appComponents = (Properties) getServlet().getServletContext().getAttribute("appComponents");
			String activePaymentProcessors = appComponents.getProperty("activePaymentProcessors");
			if (!activePaymentProcessors.contains("AuthorizeNetCIMProcessor")) {
				if (log.isInfoEnabled())
					log.info("CIM is not an active payment processor.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Check Settings
			String authorizeNetCIMHostedMethod = (String) baseForm.getSettings()
					.getValue("authorizeNetCIMHostedMethod");
			if (!"API Only".equals(authorizeNetCIMHostedMethod)) {
				if (log.isInfoEnabled())
					log.info("API Only is not turned on");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowedAPI"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			Customer customer = baseForm.getUser().getCustomer();
			Order order = baseForm.getUser().getOrder();
			AuthorizeNetCIMProcessor cimProcessor =  baseForm.getBusinessObjectFactory()
					.createObject(AuthorizeNetCIMProcessor.class);
			cimProcessor.initialize();

			// We could try to add a cim profile here if we can't find one, but
			// in theory this action should only be called after one is created.
			// Normally, during checkout, the profile is created in
			// com.softslate.commerce.customer.payment.authorizeNetCIM.AuthorizeNetCIMForm#preparePaymentFormHook()
			// which means, do not try to add payment profiles until after the
			// payment form is shown to the user.
			long cimProfileID = cimProcessor.findValidCimProfileID(customer, order);
			if (cimProfileID < 1) {
				if (log.isErrorEnabled()) {
					log.error("Can't find existing CIM profile for user.");
				}
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesGetProfileError"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Verify we have a cc number because the baseForm will not check
			// anything if it is blank because it is also used for editing
			// payment profiles. Long story.
			if (StringUtils.isBlank(baseForm.getDecryptedNumber())) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.cardNumberRequired"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			AuthorizeNetCIMResult addPaymentResult = null;

			try {
				addPaymentResult = cimProcessor.createCustomerPaymentProfile(cimProfileID,
						PropertyUtils.describe(baseForm), true);
			} catch (Exception ex) {
				if (log.isErrorEnabled()) {
					log.error("Exception adding payment profile: " + ex.getMessage());
				}
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesUnableToCreateProfile"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}
			if (addPaymentResult.getError()) {
				boolean foundMessage = false;
				logProfileErrors(addPaymentResult.getMessages());
				if (addPaymentResult.getMessages() != null && addPaymentResult.getMessages().size() > 0) {
					for (Map<String, String> message : addPaymentResult.getMessages()) {
						String code = message.get("code");
						if (StringUtils.isNotBlank(code)) {
							String msg = baseForm.getMessage("core", "errors.cimProfileError" + code, null);
							if (StringUtils.isNotBlank(msg)) {
								baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
										new ActionMessage("errors.detail", msg));
								foundMessage = true;
							}
						}
					}
				}
				if (!foundMessage) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.cimManageProfilesUnableToCreateProfile"));
				}
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			} else {
				// Add a message to tell the user it worked
				baseForm.getMessages()
						.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.cimProfileAdded"));
				saveMessages(baseForm.getRequest(), baseForm.getMessages());
				baseForm.getRequest().setAttribute("addressForm", baseForm);
				return mapping.findForward("success");
			}

		}
	}

	private void logProfileErrors(List<Map<String, String>> messages) {
		if (log.isInfoEnabled()) {
			if (messages != null) {
				for (Map<String, String> message : messages) {
					log.info(StringUtils.stripToEmpty(message.get("code")) + ": "
							+ StringUtils.stripToEmpty(message.get("text")));
				}
			}
		}
	}
}
