package com.softslate.commerce.customer.customer;

import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.order.CIMBillingAddressForm;

/**
 * 
 * @author Jason McWilliams
 * 
 */
public class AddCIMProfileFormAction extends BaseAction {
	static Log log = LogFactory.getLog(AddCIMProfileFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		CIMBillingAddressForm baseForm = (CIMBillingAddressForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		synchronized (request.getSession()) {
			Properties appComponents = (Properties) getServlet().getServletContext().getAttribute("appComponents");
			String activePaymentProcessors = appComponents.getProperty("activePaymentProcessors");
			if (!activePaymentProcessors.contains("AuthorizeNetCIMProcessor")) {
				if (log.isInfoEnabled())
					log.info("CIM is not an active payment processor.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Check Settings
			String authorizeNetCIMHostedMethod = (String) baseForm.getSettings()
					.getValue("authorizeNetCIMHostedMethod");
			if (!"API Only".equals(authorizeNetCIMHostedMethod)) {
				if (log.isInfoEnabled())
					log.info("API Only is not turned on");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowedAPI"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Pre-Pop the Add Form with Billing address in order if it exists

			Order order = baseForm.getUser().getOrder();
			if (order != null) {
				PropertyUtils.copyProperties(baseForm, order);
			}
			baseForm.getRequest().setAttribute("addressForm", baseForm);
			return mapping.findForward("success");

		}
	}
}
