package com.softslate.commerce.customer.customer;

import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import AuthNet.Rebill.CustomerAddressType;
import AuthNet.Rebill.CustomerPaymentProfileMaskedType;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMPaymentProfileMin;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMProcessor;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResult;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.order.CIMBillingAddressForm;

/**
 * 
 * @author Jason McWilliams
 * 
 */
public class EditCIMProfileAction extends BaseAction {
	static Log log = LogFactory.getLog(EditCIMProfileAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {

		if (log.isInfoEnabled())
			log.info("Starting action.");
		CIMBillingAddressForm baseForm = (CIMBillingAddressForm) form;
		synchronized (request.getSession()) {
			Properties appComponents = (Properties) getServlet().getServletContext().getAttribute("appComponents");
			String activePaymentProcessors = appComponents.getProperty("activePaymentProcessors");
			if (!activePaymentProcessors.contains("AuthorizeNetCIMProcessor")) {
				if (log.isInfoEnabled())
					log.info("CIM is not an active payment processor.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Check Settings
			String authorizeNetCIMHostedMethod = (String) baseForm.getSettings()
					.getValue("authorizeNetCIMHostedMethod");
			if (!"API Only".equals(authorizeNetCIMHostedMethod)) {
				if (log.isInfoEnabled())
					log.info("API Only is not turned on");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowedAPI"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			Customer customer = baseForm.getUser().getCustomer();
			Order order = baseForm.getUser().getOrder();
			AuthorizeNetCIMProcessor cimProcessor =  baseForm.getBusinessObjectFactory()
					.createObject(AuthorizeNetCIMProcessor.class);
			cimProcessor.initialize();
			AuthorizeNetCIMPaymentProfileMin paymentProfile = cimProcessor.findMatchingPaymentProfile(customer, order,
					baseForm.getCimPaymentProfileID());
			if (paymentProfile != null) {
				// New address
				CustomerAddressType newAddress = null;
				try {
					newAddress = cimProcessor.buildCustomerAddressType(PropertyUtils.describe(baseForm), true);
				} catch (Exception ex) {
					if (log.isErrorEnabled()) {
						log.error("Exception parsing billing address for payment profile: " + ex.getMessage());
					}
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.cimManageProfilesGetProfileError"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
					return mapping.findForward("failure");
				}
				CustomerPaymentProfileMaskedType maskedPaymentProfile = paymentProfile
						.getCustomerPaymentProfileMaskedType();
				if (maskedPaymentProfile != null) {
					maskedPaymentProfile.setBillTo(newAddress);
					AuthorizeNetCIMResult updatePaymentResult = null;
					try {
						updatePaymentResult = cimProcessor.updateCustomerPaymentProfile(paymentProfile
								.getCustomerProfileId(), maskedPaymentProfile);
					} catch (Exception ex) {
						if (log.isErrorEnabled()) {
							log.error("Exception updating payment profile: " + ex.getMessage());
						}
						baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.cimManageProfilesUnableToUpdateProfile"));
						saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
						return mapping.findForward("failure");
					}
					if (updatePaymentResult.getError()) {
						if (log.isErrorEnabled()) {
							log.error("Error updating payment profile" + updatePaymentResult.getMessages());
						}
						baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.cimManageProfilesUnableToUpdateProfile"));
						saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
						return mapping.findForward("failure");
					} else {
						// Add a message to tell the user it worked
						baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("messages.cimProfileEdited"));
						saveMessages(baseForm.getRequest(), baseForm.getMessages());
						baseForm.getRequest().setAttribute("addressForm", baseForm);
						return mapping.findForward("success");
					}
				} else {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.cimManageProfilesGetProfileError"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
					return mapping.findForward("failure");
				}
			} else {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesGetProfileError"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}
		}
	}
}
