package com.softslate.commerce.customer.customer;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class for submissions to update a customer's password. Used
 * for requests to "/UpdatePassword.do".
 * <p>
 * The <code>validate</code> method of this class checks for the existance of
 * the <code>userName</code> and <code>decryptedPassword</code> properties,
 * and checks that the <code>decryptedPassword</code> property matches the
 * <code>confirmPassword</code> property.
 * <p>
 * It also checks the length of the password against the "minimumPasswordLength"
 * database setting. If the "keepUserNameAndEmailInSync" setting is on, it will
 * verify that the user name is a valid email address and copy the user name to
 * the <code>email1</code> property.
 * 
 * @author David Tobey
 */
public class UpdatePasswordForm extends BaseForm {
	private static final long serialVersionUID = 3258131362296115257L;

	static Log log = LogFactory.getLog(UpdatePasswordForm.class);

	private String userName = null;

	public String getUserName() {
		return (this.userName);
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	private String email1 = null;

	public String getEmail1() {
		return (this.email1);
	}

	public void setEmail1(String email1) {
		this.email1 = email1;
	}

	private String decryptedPassword = null;

	public String getDecryptedPassword() {
		return (this.decryptedPassword);
	}

	public void setDecryptedPassword(String decryptedPassword) {
		this.decryptedPassword = decryptedPassword;
	}

	private String confirmPassword = null;

	public String getConfirmPassword() {
		return (this.confirmPassword);
	}

	public void setConfirmPassword(String confirmPassword) {
		this.confirmPassword = confirmPassword;
	}

	private String csrfToken = null;

	public String getCsrfToken() {
		return csrfToken;
	}

	public void setCsrfToken(String csrfToken) {
		this.csrfToken = csrfToken;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		// Check for catalog mode
		try {
			String catalogMode = (String) getSettings().getValue("catalogMode");
			if (catalogMode.equals("1")) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.catalogMode"));
				return getErrors();
			}
		} catch (Exception e) {
		}
		// Validate the user name
		if (getUserName() == null || getUserName().length() == 0) {
			String message = getMessage("customer", "customer.userName", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		// Set the minimum password length
		int minimumPasswordLength = 0;
		try {
			String minimumPasswordLengthString = (String) getSettings()
					.getValue("minimumPasswordLength");
			minimumPasswordLength = Integer
					.parseInt(minimumPasswordLengthString);
		} catch (Exception e) {
			log.error(e.toString(), e);
		}
		// Validate the password
		if (getDecryptedPassword() == null
				|| getDecryptedPassword().length() < minimumPasswordLength) {
			String message = getMessage("customer", "customer.password", null);
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.minlength", message, Integer
							.toString(minimumPasswordLength)));
		}
		// Check that the password and confirm password match.
		if (getDecryptedPassword() != null && getConfirmPassword() != null
				&& !getDecryptedPassword().equals(getConfirmPassword())) {
			String message1 = getMessage("customer", "customer.password", null);
			String message2 = getMessage("customer",
					"customer.confirmPassword", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.mustMatch", message1, message2));
		}
		// If the user name and email are to be kept in sync, copy the user name
		// over and verify it is a valid email address.
		try {
			String keepUserNameAndEmailInSync = (String) getSettings()
					.getValue("keepUserNameAndEmailInSync");
			if (keepUserNameAndEmailInSync.equals("1")) {
				setEmail1(getUserName());
				if (!formUtils().isValidEmail(getEmail1())) {
					getErrors().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.invalidEmail",
									getEmail1()));
				}
				// Otherwise, copy over the email1 property, so this form can be
				// used for
				// with logged-in customers to update their info.
			} else if (getUser().getCustomer() != null
					&& getUser().getCustomer().getEmail1() != null) {
				setEmail1(getUser().getCustomer().getEmail1());
			}
		} catch (Exception e) {
			log.error(e.toString(), e);
		}
		// Ensure that the form submission is coming from the site itself
		if (getCsrfToken() == null || getCsrfToken().length() == 0
				|| !getCsrfToken().equals(getRequest().getSession().getAttribute("csrfToken"))) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.csrfError"));
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}
}
