package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class for submissions of the checkout addresses form. Used for
 * requests to "/CheckoutAddresses.do".
 * <p>
 * The <code>execute</code> method of this class first clears the session's
 * "shippingAndTaxesProcessed" attribute. It then runs
 * {@link com.softslate.commerce.customer.core.ActionUtils#findCheckoutScreen(BaseForm)}
 * to verifiy that the user has made it this far in the checkout process.
 * Finally, it invokes
 * {@link com.softslate.commerce.customer.order.CheckoutAction#processCheckoutAddresses(BaseForm)}
 * to process the checkout addresses, shipping, and taxes.
 * 
 * @author David Tobey
 */
public class CheckoutAddressesAction extends CheckoutAction {
	static Log log = LogFactory.getLog(CheckoutAddressesAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		BaseForm baseForm = (BaseForm) form;

		synchronized (baseForm.getRequest().getSession()) {
			// Make sure the user is allowed to have this action performed.
			baseForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "false");
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			String checkoutScreen = actionUtils.findCheckoutScreen(baseForm);
			if (!(checkoutScreen.equals("checkoutAddresses") || checkoutScreen.equals("checkoutAjaxOnePage") || checkoutScreen
					.equals("checkoutShipping"))) {
				return baseForm.getMapping().findForward(checkoutScreen);
			}
			// Process the form.
			return processCheckoutAddresses(baseForm, response);
		}
	}
}
