package com.softslate.commerce.customer.order;

import java.lang.reflect.Method;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.util.RequestUtils;

import com.softslate.commerce.businessobjects.customer.CustomerAddressProcessor;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.businessobjects.order.CartDiscountProcessor;
import com.softslate.commerce.businessobjects.order.CartProcessor;
import com.softslate.commerce.businessobjects.order.InventoryProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderDelivery;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.payment.PaymentProcessor;
import com.softslate.commerce.businessobjects.shipping.AddressValidationProcessor;
import com.softslate.commerce.businessobjects.shipping.BasicShippingProcessor;
import com.softslate.commerce.businessobjects.shipping.ShippingProcessor;
import com.softslate.commerce.businessobjects.shipping.ShippingRuleProcessor;
import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.customer.customer.LoginForm;
import com.softslate.commerce.customer.customer.RegisterForm;
import com.softslate.emailer.businessobjects.core.EmailerProcessor;

/**
 * <code>CheckoutUtilsImpl</code> is the default implementation of the
 * <code>CheckoutUtils</code> interface for the application.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.customer.order.CheckoutUtils
 * 
 */
public class CheckoutUtilsImpl implements CheckoutUtils {

	static Log log = LogFactory.getLog(CheckoutUtilsImpl.class);

	public String findCheckoutScreen(BaseForm baseForm) throws Exception {
		// Check for catalog mode
		String catalogMode = (String) baseForm.getSettings().getValue(
				"catalogMode");
		if (catalogMode.equals("1")) {
			if (log.isDebugEnabled())
				log.debug("Catalog mode. Checking out is not allowed.");
			return "catalogMode";
		}
		// If the cart is empty, forward to the cart page where the user
		// sees an empty cart message.
		if (log.isDebugEnabled())
			log.debug("Checking for empty cart.");
		if (baseForm.getUser().getOrder() == null
				|| baseForm.getUser().getOrder().getQuantity() == null
				|| baseForm.getUser().getOrder().getQuantity().intValue() == 0) {
			if (log.isDebugEnabled())
				log.debug("Cart's empty. Forwarding to the cart screen.");
			return "cart";
		}
		// If the store is configured to force logins AND the user is
		// not logged in, go to the force login screen.
		String forceLogin = (String) baseForm.getSettings().getValue(
				"forceLogin");
		if (forceLogin.equals("1") && baseForm.getUser().getCustomer() == null) {
			if (log.isDebugEnabled())
				log.debug("Forwarding to the force login screen.");
			return "forceLogin";
		}
		// If the store is configured to invite the user to login
		// AND the user is not logged in
		// AND the user has not chosen to order without an account, go to
		// the login/create account/order without an account screen
		String inviteLoginBeforeCheckout = (String) baseForm.getSettings()
				.getValue("inviteLoginBeforeCheckout");
		if (inviteLoginBeforeCheckout.equals("1")
				&& baseForm.getUser().getCustomer() == null
				&& (String) baseForm.getRequest().getSession()
						.getAttribute("declinedLoginBeforeCheckout") == null) {
			if (log.isDebugEnabled())
				log.debug("Forwarding to the invite login screen.");
			return "inviteLogin";
		}
		// If the required billing or delivery fields are not in the session,
		// send the user to the address form.
		// Skip if we have a PayPal PayerID
		if (log.isDebugEnabled())
			log.debug("Checking for required address fields.");
		String shippingAndTaxesProcessed = (String) baseForm.getRequest()
				.getSession().getAttribute("shippingAndTaxesProcessed");
		String useAjaxOnePageCheckout = (String) baseForm.getSettings().getValue("useAjaxOnePageCheckout");
		String useCheckoutCombo = (String) baseForm.getSettings().getValue(
				"useCheckoutCombo");
		String shippingRequired = (String) baseForm.getSettings().getValue(
				"shippingRequired");
		String combineCheckoutShippingAddresses = (String) baseForm
				.getSettings().getValue("combineCheckoutShippingAddresses");
		String paypalPayerID = null;
		if (baseForm.getUser().getCustomMap() != null) {
			paypalPayerID = (String) baseForm.getUser().getCustomMap()
					.get("paypalPayerID");
		}
		if (paypalPayerID == null
				&& (!validateBillingFields(baseForm) || (shippingRequired
						.equals("1") && !validateDeliveryFields(baseForm)))) {
			if (log.isDebugEnabled())
				log.debug("Not all required address fields in session.");
			if (useAjaxOnePageCheckout != null && useAjaxOnePageCheckout.equals("1")) {
				return "checkoutAjaxOnePage";
			}
			if (useCheckoutCombo.equals("1")) {
				return "checkoutCombo";
			}
			if (combineCheckoutShippingAddresses.equals("1")) {
				return "checkoutComboShipping";
			}
			return "checkoutAddresses";
		}
		// If the user's shipping and taxes have not been processed,
		// send the user to the address or shipping form.
		if (shippingAndTaxesProcessed == null
				|| !shippingAndTaxesProcessed.equals("true")) {
			if (log.isDebugEnabled())
				log.debug("Shipping and taxes not processed.");
			if (paypalPayerID != null && shippingRequired.equals("1")) {
				return "checkoutShipping";
			}
			if (useAjaxOnePageCheckout.equals("1")) {
				return "checkoutAjaxOnePage";
			}
			if (useCheckoutCombo.equals("1")) {
				return "checkoutCombo";
			}
			if (combineCheckoutShippingAddresses.equals("1")) {
				return "checkoutComboShipping";
			}
			if (shippingRequired.equals("1")) {
				return "checkoutShipping";
			}
			return "checkoutAddresses";
		}
		// If we have not collected payment information from the user and
		// payment information is required, send the user to the checkout
		// payment or checkout combo screen.
		String paymentRequired = (String) baseForm.getSettings().getValue(
				"paymentRequired");
		if (paymentRequired.equals("1")
				&& paypalPayerID == null
				&& (baseForm.getRequest().getSession()
						.getAttribute("validatedPaymentForm") == null || baseForm
						.getRequest().getSession()
						.getAttribute("processorName") == null)) {
			// If the combo screen is in use, just check that the subtotal > 0,
			// otherwise, check the total
			if (useAjaxOnePageCheckout.equals("1") && baseForm.getUser().getOrder().getSubtotal().doubleValue() > 0) {
				return "checkoutAjaxOnePage";
			} else if (useCheckoutCombo.equals("1")
					&& baseForm.getUser().getOrder().getSubtotal()
							.doubleValue() > 0) {
				if (log.isDebugEnabled())
					log.debug("Forwarding to the combo screen.");
				return "checkoutCombo";
			} else if (baseForm.getUser().getOrder().getTotal().doubleValue() > 0) {
				if (log.isDebugEnabled())
					log.debug("Forwarding to the payment forms screen.");
				return "checkoutPayment";
			}
		}
		// The only screen left is the confirm screen.
		if (log.isDebugEnabled())
			log.debug("Checkout confirm screen.");
		return "checkoutConfirm";
	}

	public boolean validateBillingFields(BaseForm baseForm) throws Exception {
		// Loop through each of the required billing fields checking to see if
		// each is here
		String billingRequired = (String) baseForm.getSettings().getValue(
				"billingRequired");
		StringTokenizer billingRequiredList = new StringTokenizer(
				billingRequired, ",");
		Class classDefinition = baseForm.getUser().getOrder().getClass();
		Class[] argumentTypes = null;
		Object[] arguments = null;
		while (billingRequiredList.hasMoreTokens()) {
			String thisMethodString = "get"
					+ baseForm.formUtils().capitalize(
							billingRequiredList.nextToken());
			Method method = classDefinition.getMethod(thisMethodString,
					argumentTypes);
			String thisValue = (String) method.invoke(baseForm.getUser()
					.getOrder(), arguments);
			if (thisValue == null) {
				return false;
			}
		}
		return true;
	}

	public boolean validateDeliveryFields(BaseForm baseForm) throws Exception {
		// Set up arguments.
		Class[] argumentTypes = null;
		Object[] arguments = new Object[] {};
		// Loop through each of the deliveries in memory.
		Collection deliveryCollection = baseForm.getUser().getOrder()
				.getOrderDeliveryCollection();
		Iterator iterator1 = deliveryCollection.iterator();
		while (iterator1.hasNext()) {
			OrderDelivery odb = (OrderDelivery) iterator1.next();
			Class classDefinition = odb.getClass();
			// Loop through each of the required delivery fields checking to see
			// if each is here
			String deliveryRequired = (String) baseForm.getSettings().getValue(
					"deliveryRequired");
			StringTokenizer deliveryRequiredList = new StringTokenizer(
					deliveryRequired, ",");
			while (deliveryRequiredList.hasMoreTokens()) {
				String thisMethodString = "get"
						+ baseForm.formUtils().capitalize(
								deliveryRequiredList.nextToken());
				Method method = classDefinition.getMethod(thisMethodString,
						argumentTypes);
				String thisValue = (String) method.invoke(odb, arguments);
				if (thisValue == null) {
					return false;
				}
			}
		}
		return true;
	}

	public void prepareLoginForm(BaseForm baseForm, ActionServlet servlet)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Preparing the login form.");
		// Create a LoginForm instance and put it in the request scope.
		LoginForm loginForm = new LoginForm();
		loginForm.setServlet(servlet);
		RequestUtils.populate(loginForm, baseForm.getRequest());
		baseForm.getRequest().setAttribute("loginForm", loginForm);
		return;
	}

	public void prepareRegisterForm(BaseForm baseForm, ActionServlet servlet)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Preparing the register form.");
		// Create a RegisterForm instance and put it in the request scope.
		RegisterForm registerForm = new RegisterForm();
		registerForm.setServlet(servlet);
		RequestUtils.populate(registerForm, baseForm.getRequest());
		baseForm.getRequest().setAttribute("registerForm", registerForm);
		return;
	}

	public ActionForward processCheckoutLogin(BaseForm baseForm,
			HttpServletResponse response) throws Exception {

		// Prepare the autoLoginToken, if called for
		boolean setAutoLoginCookie = false;
		Map parameters = PropertyUtils.describe(baseForm);
		String allowCustomerAutoLogins = (String) baseForm.getSettings()
				.getValue("allowCustomerAutoLogins");
		if (parameters.get("rememberMe") != null
				&& !parameters.get("rememberMe").equals("false")
				&& allowCustomerAutoLogins != null
				&& allowCustomerAutoLogins.equals("1")) {
			setAutoLoginCookie = true;
			parameters.put("autoLoginToken", baseForm.getRequest().getSession()
					.getId());
		}

		// Process the login
		CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"customerProcessorImplementer");
		Map result = customerProcessor.processLogin(parameters);

		// Process errors
		if (result != null && result.get("resultCode") != null
				&& !result.get("resultCode").equals("0")) {
			String resultCode = (String) result.get("resultCode");
			// Act successfully if user is already logged in
			if (!resultCode.equals("errors.userAlreadyLoggedIn")) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage(resultCode));
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return baseForm.getMapping().findForward("failureInvalidLogin");
			}
		}

		// Set the autoLoginToken cookie
		if (setAutoLoginCookie) {
			String autoLoginDays = (String) baseForm.getSettings().getValue(
					"autoLoginDays");
			int maxAge = 86400000; // Default to 10000 days
			if (autoLoginDays != null && !autoLoginDays.equals("0")) {
				try {
					maxAge = Integer.parseInt(autoLoginDays) * 86400;
				} catch (NumberFormatException nfe) {
					if (log.isInfoEnabled())
						log.info(nfe);
				}
			}
			Cookie autoLoginCookie = new Cookie("sscaltoken", baseForm
					.getUser().getCustomer().getAutoLoginToken());
			autoLoginCookie.setMaxAge(maxAge);
			response.addCookie(autoLoginCookie);
		}

		// Logging in may have affected discounts, thus we need to make sure
		// shipping and taxes are reprocessed.
		baseForm.getRequest().getSession()
				.setAttribute("shippingAndTaxesProcessed", "false");
		return baseForm.getMapping().findForward("success");
	}

	public ActionForward processCheckoutRegister(BaseForm baseForm)
			throws Exception {
		CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"customerProcessorImplementer");
		Map result = customerProcessor.processRegister(PropertyUtils
				.describe(baseForm));
		// Process errors
		if (result != null && result.get("resultCode") != null
				&& !result.get("resultCode").equals("0")) {
			String resultCode = (String) result.get("resultCode");
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage(resultCode));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Error(s): " + baseForm.getErrors().toString());
			return baseForm.getMapping().findForward("failureUserName");
		}
		// Logging in may have affected discounts, thus we need to make sure
		// shipping and taxes are reprocessed.
		baseForm.getRequest().getSession()
				.setAttribute("shippingAndTaxesProcessed", "false");
		// Process the Emailer integration
		try {
			Map parameters = PropertyUtils.describe(baseForm);
			parameters.put("Email",
					PropertyUtils.getProperty(baseForm, "userName"));
			EmailerProcessor processor = (EmailerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"emailerProcessorImplementer");
			processor.processCustomerRegistration(parameters);
		} catch (Exception e) {
			log.warn("Error signing up user with Emailer: ", e);
		}
		return baseForm.getMapping().findForward("success");
	}

	public ActionForward processDeclineLogin(BaseForm baseForm)
			throws Exception {
		baseForm.getRequest().getSession()
				.setAttribute("declinedLoginBeforeCheckout", "true");
		return baseForm.getMapping().findForward("success");
	}

	public Collection loadShippingOptions(BaseForm baseForm) throws Exception {
		// Load the shipping options for the shipping form
		Collection shippingOptions = baseForm
				.getBusinessObjectFactory()
				.invokeMultipleCollectionProcessors("activeShippingProcessors",
						"loadShippingOptions", PropertyUtils.describe(baseForm));
		// Use shipping rule processor to reduce the possible methods available
		// depending on shipping rules setup by the admin.
		ShippingRuleProcessor shippingRuleProcessor = (ShippingRuleProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"shippingRuleProcessorImplementer");
		int orderID = 0;
		if (baseForm.getUser() != null && baseForm.getUser().getOrder() != null
				&& baseForm.getUser().getOrder().getOrderID() > 0) {
			orderID = baseForm.getUser().getOrder().getOrderID();
		}
		shippingOptions = shippingRuleProcessor
				.findAvailableShippingProcessors(shippingOptions, orderID);

		// Use the discount processor to apply shipping discounts to
		// each shipping option.
		CartDiscountProcessor discountProcessor = (CartDiscountProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"cartDiscountProcessorImplementer");
		shippingOptions = discountProcessor
				.processShippingDiscountsOnShippingOptions(shippingOptions);

		return shippingOptions;
	}

	public Map findShippingOptionFromOrder(Collection options, Order order,
			BaseForm baseForm) {

		if (options == null || order == null || order.firstDelivery() == null) {
			return null;
		}
		String shippingMethod = order.getShippingMethod();
		String statusDetails = order.firstDelivery().getStatusDetails();
		Map previousOption = null;
		if ((shippingMethod != null && shippingMethod.length() > 0)
				|| (statusDetails != null && statusDetails.length() > 0)) {

			// First try to find it by its name
			String storeShippingMethodNameWithOrder = (String) baseForm
					.getSettings().getValue("storeShippingMethodNameWithOrder");
			if (storeShippingMethodNameWithOrder != null
					&& storeShippingMethodNameWithOrder.equals("1")) {
				Iterator i = options.iterator();
				while (i.hasNext()) {
					Map option = (Map) i.next();
					String name = (String) option.get("name");
					if (shippingMethod != null && shippingMethod.equals(name)) {
						previousOption = option;
						break;
					}
				}
			}

			// If that failed then try to find it by its code
			if (previousOption == null) {
				Iterator i = options.iterator();
				while (i.hasNext()) {
					Map option = (Map) i.next();
					String code = (String) option.get("code");
					if ((shippingMethod != null && shippingMethod.equals(code))
							|| ((shippingMethod == null || shippingMethod
									.length() == 0) && statusDetails != null && statusDetails
									.equals(code))) {
						previousOption = option;
						break;
					}
				}

			}
		}
		return previousOption;
	}

	public void prepareCheckoutAddressesAddressBookForm(CheckoutAddressesForm baseForm) throws Exception {
		prepareCheckoutAddressesForm(baseForm, true, true);
	}

	public void prepareCheckoutAddressesForm(CheckoutAddressesForm baseForm) throws Exception {
		prepareCheckoutAddressesForm(baseForm, false, false);
	}

	public void prepareCheckoutAddressesForm(CheckoutAddressesForm baseForm, boolean skipDeliveryFields,
			boolean skipBillingFields)
			throws Exception {
		// If the form was just submitted, leave it alone, otherwise, populate
		// it from the current order, if the user has passed through the address
		// form already
		String shippingAndTaxesProcessed = (String) baseForm.getRequest().getSession()
				.getAttribute("shippingAndTaxesProcessed");
		if (baseForm.getRequest().getAttribute("taxProcessingError") != null
				|| (!baseForm.isSubmitted() && shippingAndTaxesProcessed != null
						&& shippingAndTaxesProcessed.equals("true") && baseForm.getUser().getOrder() != null)) {
			if (log.isDebugEnabled())
				log.debug("Populating addressForm from the current order.");
			if (!skipBillingFields) {
				BeanUtils.copyProperties(baseForm, baseForm.getUser().getOrder());
			}
			if (!skipDeliveryFields && baseForm.getUser().getOrder().firstDelivery() != null) {
				CustomerAddressProcessor cap = baseForm.getBusinessObjectFactory().createObject(
						CustomerAddressProcessor.class);
				BeanUtils.copyProperties(baseForm, cap.describeDeliveryOrBillingForAddressBook(
						PropertyUtils.describe(baseForm.getUser().getOrder().firstDelivery()), "delivery", false));
			}
			// Set up the state and 'Other' fields for presentation.
			baseForm.prepareStateAndOtherFields(baseForm.getUser().getOrder());
			// We have to reinitialize the other properties of the base form.
			baseForm.initializeProperties(baseForm.getMapping(), baseForm.getRequest());
		}
		// If the form was just submitted, leave it alone, otherwise, populate
		// it from a logged in user's info.
		else if (!baseForm.isSubmitted() && baseForm.getUser().getCustomer() != null) {
			if (log.isDebugEnabled())
				log.debug("Populating addressForm from a logged in customer.");

			if (!skipBillingFields) {
				BeanUtils.copyProperties(baseForm, baseForm.getUser().getCustomer());
				if ((baseForm.getBillingEmail1() == null || baseForm.getBillingEmail1().length() == 0)
						&& baseForm.formUtils().isValidEmail(baseForm.getUser().getCustomer().getUserName())) {
					baseForm.setBillingEmail1(baseForm.getUser().getCustomer().getUserName());
				}
			}

			if (!skipDeliveryFields && baseForm.getUser().getCustomer().getFirstAddress() != null) {
				CustomerAddressProcessor cap = baseForm.getBusinessObjectFactory().createObject(
						CustomerAddressProcessor.class);
				BeanUtils.copyProperties(baseForm, cap.describeDeliveryOrBillingForAddressBook(
						PropertyUtils.describe(baseForm.getUser().getCustomer().getFirstAddress()), "delivery", false));
				if ((baseForm.getDeliveryEmail1() == null || baseForm.getDeliveryEmail1().length() == 0)
						&& baseForm.formUtils().isValidEmail(baseForm.getUser().getCustomer().getUserName())) {
					baseForm.setDeliveryEmail1(baseForm.getUser().getCustomer().getUserName());
				}
			}
			// Set up the state and 'Other' fields for presentation.
			baseForm.prepareStateAndOtherFields(baseForm.getUser().getCustomer());
			// We have to reinitialize the other properties of the base form.
			baseForm.initializeProperties(baseForm.getMapping(), baseForm.getRequest());
		}
		// Rename so the jsp's can be generalized.
		baseForm.getRequest().setAttribute("addressForm", baseForm);
		return;
	}

	public void prepareCheckoutShippingForm(BaseForm baseForm) throws Exception {
		// If we require shipping but only allow one shipment, include the
		// shipping selection form on the address screen.
		if (baseForm.getSettings().getValue("shippingRequired")
						.equals("1")) {
			baseForm.getRequest().setAttribute("shippingForm", baseForm);
			Collection shippingOptions = loadShippingOptions(baseForm);
			baseForm.getRequest().setAttribute("shippingOptions",
					shippingOptions);

			// Find the previously select option, if any, and prepopulate the
			// form with it.
			String optionFromForm = (String) PropertyUtils.getProperty(
					baseForm, "shippingOption");
			if (optionFromForm == null || optionFromForm.length() == 0) {
				Map selectedOption = findShippingOptionFromOrder(
						shippingOptions, baseForm.getUser().getOrder(),
						baseForm);
				if (selectedOption != null) {
					String selectedOptionValue = selectedOption.get(
							"processorClass").toString()
							+ "|" + selectedOption.get("code").toString();
					PropertyUtils.setProperty(baseForm, "shippingOption",
							selectedOptionValue);
					baseForm.getRequest().setAttribute(
							"selectedShippingOption", selectedOption);
				}
			}
		}
		return;
	}

	public ActionForward processCheckoutAddresses(BaseForm baseForm)
			throws Exception {
		return processCheckoutAddresses(baseForm, null);
	}

	public ActionForward processCheckoutAddresses(BaseForm baseForm,
			HttpServletResponse response) throws Exception {

		// Since personal information is about to be stored with the cart,
		// remove the autoSavedCartToken from the order and delete the cookie
		// from the browser
		Map parameters = PropertyUtils.describe(baseForm);
		while (parameters.values().remove(null))
			;
		parameters.put("autoSavedCartToken", null);
		if (response != null) {
			Cookie[] cookies = baseForm.getRequest().getCookies();
			if (cookies != null) {
				for (int i = 0; i < cookies.length; i++) {
					if (cookies[i] != null && cookies[i].getName() != null
							&& cookies[i].getName().equals("sscsctoken")) {
						Cookie autoSavedCartCookie = cookies[i];
						autoSavedCartCookie.setMaxAge(0);
						response.addCookie(autoSavedCartCookie);
						break;
					}
				}
			}
		}
		// Call CartProcessor.processCheckoutAddresses()
		CartProcessor cartProcessor = (CartProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"cartProcessorImplementer");
		cartProcessor.processCheckoutAddresses(parameters);
		if (log.isDebugEnabled())
			log.debug("Saved the addresses in the cart.");

		// Process inventory and discounting check
		String trackInventory = (String) baseForm.getSettings().getValue(
				"trackInventory");
		String applyDiscounts = (String) baseForm.getSettings().getValue(
				"applyDiscounts");
		if ((trackInventory != null && trackInventory.equals("1"))
				|| (applyDiscounts != null && applyDiscounts.equals("1"))) {
			Map invDiscResults = processInventoryDiscountCheck(baseForm);
			int resultCode = Integer.parseInt((String) invDiscResults
					.get("resultCode"));
			if (resultCode == 0) {
				if (log.isDebugEnabled())
					log.debug("Processed the inventory and discounting check.");
			} else if (resultCode == 1) {
				if (log.isDebugEnabled())
					log.debug("Some order items adjusted - must inform user.");
				return baseForm.getMapping()
						.findForward("failureItemsAdjusted");
			} else if (resultCode == 2) {
				if (log.isDebugEnabled())
					log.debug("All order items removed due to inventory");
				return baseForm.getMapping().findForward(
						"failureAllItemsRemoved");
			}
		}

		// Validate the address with FedEx if called for by the settings
		Map results = validateAddress(baseForm);
		if (results != null && results.get("code") != null && !results.get("code").equals("0")) {
			baseForm.getRequest().getSession().setAttribute("shippingAndTaxesProcessed", "false");
			baseForm.getUser().getOrder().firstDelivery().setIsResidential(null);
			return baseForm.getMapping().findForward("failureAddressValidation");
		} else if (results != null && results.get("addressIsResidential") != null) {
			baseForm.getUser().getOrder().firstDelivery()
					.setIsResidential((Boolean) results.get("addressIsResidential"));
		}

		// Process taxes if shipping is not required, otherwise it'll be
		// processed after the shipping.
		String shippingRequired = (String) baseForm.getSettings().getValue(
				"shippingRequired");
		if (!shippingRequired.equals("1")) {
			Map taxResults = processTax(baseForm);
			if (Integer.parseInt((String) taxResults.get("resultCode")) == 0) {
				if (log.isDebugEnabled())
					log.debug("Processed the tax.");
			} else {
				if (log.isDebugEnabled())
					log.debug("Failed to process tax.");
				if (taxResults.get("resultCode") != null) {
					// Place error messages in the request.
					String message = baseForm.getMessage("order", "tax.Error"
							+ taxResults.get("resultCode"), null);
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail", message));
					saveErrors(baseForm.getRequest(),
							(ActionMessages) baseForm.getErrors());
				}
				return baseForm.getMapping().findForward("failureTaxProcess");
			}
			baseForm.getRequest().getSession()
					.setAttribute("shippingAndTaxesProcessed", "true");
			// Reprocess discounts so gift certificates may be calculated
			// against the order total
			CartDiscountProcessor dp = (CartDiscountProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"cartDiscountProcessorImplementer");
			Map discountResults = dp.processAfterTaxes(PropertyUtils
					.describe(baseForm));
			if (log.isDebugEnabled() && discountResults != null) {
				log.debug("Discount results: " + discountResults);
			}
		}

		// Process the Emailer integration
		try {
			parameters.put("Email",
					PropertyUtils.getProperty(baseForm, "billingEmail1"));
			parameters.put("First Name",
					PropertyUtils.getProperty(baseForm, "billingFirstName"));
			parameters.put("Last Name",
					PropertyUtils.getProperty(baseForm, "billingLastName"));
			EmailerProcessor processor = (EmailerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"emailerProcessorImplementer");
			processor.processCheckoutAddresses(parameters);
		} catch (Exception e) {
			log.info("Cannot sign up with Emailer: " + e.toString());
		}

		// Determine the action forward to return
		if (shippingRequired.equals("1")) {
			if (log.isDebugEnabled())
				log.debug("Success--shipping required.");
			return baseForm.getMapping().findForward("success");
		} else {
			String paymentRequired = (String) baseForm.getSettings().getValue(
					"paymentRequired");
			String useCheckoutCombo = (String) baseForm.getSettings().getValue(
					"useCheckoutCombo");
			String paypalPayerID = null;
			if (baseForm.getUser().getCustomMap() != null) {
				paypalPayerID = (String) baseForm.getUser().getCustomMap()
						.get("paypalPayerID");
			}
			if (paymentRequired.equals("1")
					&& paypalPayerID == null
					&& ((baseForm.getUser().getOrder().getTotal() != null && baseForm
							.getUser().getOrder().getTotal().doubleValue() > 0) || (baseForm
							.getUser().getOrder().getSubtotal() != null
							&& baseForm.getUser().getOrder().getSubtotal()
									.doubleValue() > 0 && useCheckoutCombo
							.equals("1")))) {
				if (log.isDebugEnabled())
					log.debug("Success--payment required but no shipping.");
				return baseForm.getMapping().findForward(
						"successNoShippingRequired");
			} else {
				String displayCheckoutConfirm = (String) baseForm.getSettings()
						.getValue("displayCheckoutConfirm");
				if (displayCheckoutConfirm.equals("1") || paypalPayerID != null) {
					if (log.isDebugEnabled())
						log.debug("Success--no payment required but must confirm.");
					return baseForm.getMapping().findForward(
							"successNoPaymentRequiredConfirm");
				} else {
					// Put the user bean in the request scope to avoid another
					// request changing it.
					if (log.isDebugEnabled())
						log.debug("Success--no payment required.");
					processOrderComplete(baseForm);
					return baseForm.getMapping().findForward(
							"successNoPaymentRequired");
				}
			}
		}
	}

	public Map validateAddress(BaseForm baseForm) {
		Map results = new HashMap();
		Map validationResults = null;
		try {
			AddressValidationProcessor avp = baseForm.getBusinessObjectFactory().createObject(
					AddressValidationProcessor.class);
			validationResults = avp.validateAddress(null);
			if (log.isInfoEnabled())
				log.info("validationResults: " + validationResults);
		} catch (Exception e) {
			log.error("FedEx address validation exception (ignoring): ", e);
		}

		// What if we have errors?
		if (validationResults != null && validationResults.get("errors") != null) {
			Collection errors = (Collection) validationResults.get("errors");
			if (errors.size() > 0) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.addressValidation", errors.toArray()[0]));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				results.put("code", "1");
				return results;
			}
		}

		// Return an error if the address was not validated.
		if (validationResults != null && validationResults.get("Score") != null
				&& validationResults.get("Score").toString().equals("0")) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.addressValidation", ""));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			results.put("code", "2");
			return results;
		}

		// Also send modifications back to the customer to fix.
		if (validationResults != null && validationResults.get("MODIFIED_TO_ACHIEVE_MATCH") != null
				&& validationResults.get("MODIFIED_TO_ACHIEVE_MATCH").equals(Boolean.TRUE)) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.addressValidation", ""));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			results.put("code", "3");
			return results;
		}

		// Set the residential flag
		if (validationResults != null && validationResults.get("ResidentialStatus") != null
				&& validationResults.get("ResidentialStatus").equals("BUSINESS")) {
			results.put("addressIsResidential", new Boolean(false));
		} else if (validationResults != null && validationResults.get("ResidentialStatus") != null
				&& validationResults.get("ResidentialStatus").equals("RESIDENTIAL")) {
			results.put("addressIsResidential", new Boolean(true));
		}
		results.put("code", "0");
		return results;
	}

	public Map processInventoryDiscountCheck(BaseForm baseForm)
			throws Exception {
		String resultCode = "0";
		Map results = new HashMap();
		String trackInventory = (String) baseForm.getSettings().getValue(
				"trackInventory");
		if (trackInventory.equals("1")) {
			boolean cartChanged = false;
			InventoryProcessor invProcessor = (InventoryProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"inventoryProcessorImplementer");
			results = invProcessor.processOnCheckout(PropertyUtils
					.describe(baseForm));
			if (results == null) {
				results = new HashMap();
			}
			Collection rejectedOrderItems = (Collection) results
					.get("rejectedOrderItems");

			// Add errors for each OOS sku
			if (rejectedOrderItems != null && !rejectedOrderItems.isEmpty()) {
				if (log.isDebugEnabled())
					log.debug("Adding errors for OOS items.");
				cartChanged = true;
				Iterator j = rejectedOrderItems.iterator();
				while (j.hasNext()) {
					OrderItem oi = (OrderItem) j.next();
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.oosSku", oi.getCode()));
				}
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
			}

			// Add message for each adjusted order item
			Collection adjustedOrderItems = (Collection) results
					.get("adjustedOrderItems");
			if (adjustedOrderItems != null && !adjustedOrderItems.isEmpty()) {
				cartChanged = true;
				Iterator j = adjustedOrderItems.iterator();
				while (j.hasNext()) {
					OrderItem oi = (OrderItem) j.next();
					baseForm.getMessages().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("messages.adjustedOrderItem", oi
									.getCode(), Integer.toString(oi
									.getQuantity())));
				}
				saveMessages(baseForm.getRequest(),
						(ActionMessages) baseForm.getMessages());
			}
			// If the cart changes, processing discounts to account for changes
			if (cartChanged) {

				// Reprocess discounts
				CartDiscountProcessor dp = (CartDiscountProcessor) baseForm
						.getBusinessObjectFactory().createObject(
								"cartDiscountProcessorImplementer");
				Map discountResults = dp.processOnCheckout(PropertyUtils
						.describe(baseForm));
				if (discountResults != null) {
					results.putAll(discountResults);
				}

				// Call CartProcessor hook method for all cart changes
				CartProcessor cp = (CartProcessor) baseForm
						.getBusinessObjectFactory().createObject(
								"cartProcessorImplementer");
				cp.postProcessCartChanged();
			}
			// Figure the result code
			if (cartChanged) {
				resultCode = "1";
			}
			if (baseForm.getUser().getOrder().firstDelivery()
					.getOrderItemCollection().isEmpty()) {
				resultCode = "2";
			}
		}
		results.put("resultCode", resultCode);
		return results;
	}

	public ActionForward processCheckoutShipping(BaseForm baseForm)
			throws Exception {
		// Process the shipping
		String shippingRequired = (String) baseForm.getSettings().getValue(
				"shippingRequired");
		if (shippingRequired.equals("1")) {
			Map shipResults = processShipping(baseForm);
			if (Integer.parseInt((String) shipResults.get("resultCode")) == 0) {
				if (log.isDebugEnabled())
					log.debug("Processed the shipping.");
			} else {
				if (log.isDebugEnabled())
					log.debug("Failed to process shipping.");
				return baseForm.getMapping().findForward(
						"failureShippingProcess");
			}
		}
		// Process the tax
		Map taxResults = processTax(baseForm);
		if (taxResults != null && taxResults.get("resultCode") != null
				&& Integer.parseInt((String) taxResults.get("resultCode")) == 0) {
			if (log.isDebugEnabled())
				log.debug("Processed the tax.");
		} else {
			if (log.isDebugEnabled())
				log.debug("Failed to process tax.");
			if (taxResults.get("resultCode") != null) {
				// Place error messages in the request.
				String message = baseForm.getMessage("order", "tax.Error"
						+ taxResults.get("resultCode"), null);
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.detail", message));
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
				// For AvaTax Address Validation, Check where to send them.
				if (taxResults.get("resultCode").equals("10001")) {
					if (log.isInfoEnabled())
						log.info("Failed tax address validation.");
					baseForm.getRequest().setAttribute("taxProcessingError",
							Boolean.TRUE);
					return baseForm.getMapping().findForward(
							"failureTaxAddressValidation");
				}
			}
			return baseForm.getMapping().findForward("failureTaxProcess");
		}
		baseForm.getRequest().getSession()
				.setAttribute("shippingAndTaxesProcessed", "true");
		// Reprocess discounts so gift certificates may be calculated against
		// the order total
		CartDiscountProcessor dp = (CartDiscountProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"cartDiscountProcessorImplementer");
		Map discountResults = dp.processAfterTaxes(PropertyUtils
				.describe(baseForm));
		if (log.isDebugEnabled() && discountResults != null) {
			log.debug("Discount results: " + discountResults);
		}
		// Determine action forward to return
		String paymentRequired = (String) baseForm.getSettings().getValue(
				"paymentRequired");
		String useCheckoutCombo = (String) baseForm.getSettings().getValue(
				"useCheckoutCombo");
		String paypalPayerID = null;
		if (baseForm.getUser().getCustomMap() != null) {
			paypalPayerID = (String) baseForm.getUser().getCustomMap()
					.get("paypalPayerID");
		}
		if (paymentRequired.equals("1")
				&& paypalPayerID == null
				&& (baseForm.getUser().getOrder().getTotal().doubleValue() > 0 || (baseForm
						.getUser().getOrder().getSubtotal().doubleValue() > 0 && useCheckoutCombo
						.equals("1")))) {
			if (log.isDebugEnabled())
				log.debug("Success--payment required.");
			return baseForm.getMapping().findForward("success");
		} else {
			String displayCheckoutConfirm = (String) baseForm.getSettings()
					.getValue("displayCheckoutConfirm");
			if (displayCheckoutConfirm.equals("1") || paypalPayerID != null) {
				if (log.isDebugEnabled())
					log.debug("Success--no payment required but must confirm.");
				return baseForm.getMapping().findForward(
						"successNoPaymentRequiredConfirm");
			} else {
				// Put the user bean in the request scope to avoid another
				// request changing it.
				if (log.isDebugEnabled())
					log.debug("Success--no payment required.");
				processOrderComplete(baseForm);
				return baseForm.getMapping().findForward(
						"successNoPaymentRequired");
			}
		}
	}

	public Map processShipping(BaseForm baseForm) throws Exception {

		// Parse out the processor and option that were selected
		Map results = new HashMap();
		String selectedOption = BeanUtils.getProperty(baseForm,
				"shippingOption");
		if (log.isDebugEnabled())
			log.debug("Shipping Option Selected: " + selectedOption);

		// Attempt to handle customized JSPs elegantly
		String processor = "BasicShippingProcessor";
		String option = selectedOption;
		if (selectedOption.indexOf("|") != -1) {
			String[] processorAndOption = selectedOption.split("\\|");
			if (log.isDebugEnabled())
				log.debug("Shipping Option Selected: " + processorAndOption[0]
						+ " " + processorAndOption[1]);
			processor = processorAndOption[0];
			option = processorAndOption[1];
		}

		// Loop over the activeShippingProcessors
		String activeShippingProcessors = ((Properties) baseForm.getRequest()
				.getSession().getServletContext().getAttribute("appComponents"))
				.getProperty("activeShippingProcessors");
		String[] processors = activeShippingProcessors.split(",");
		if (log.isDebugEnabled())
			log.debug("activeShippingProcessors: " + activeShippingProcessors);

		String className = "";
		for (int i = 0; i < processors.length; i++) {
			if (processors[i].endsWith("." + processor)) {
				className = processors[i];
				break;
			}
		}
		if (log.isDebugEnabled())
			log.debug("className: " + className);

		if (className.length() > 0 && option != null && option.length() > 0) {
			// Invoke
			try {
				// Send the processor a simple map instead of all the baseForm
				// fields. We do this
				// so the processor doesnt have to change to find the new |
				// delimited class/method.
				Map args = new HashMap();
				args.put("shippingOption", option);
				ShippingProcessor shippingProcessor = (ShippingProcessor) baseForm
						.getBusinessObjectFactory().createObjectFromClassName(
								className);

				// First grab the options
				Collection shippingOptions = shippingProcessor
						.loadShippingOptions(null);

				// Always load the contingency option
				BasicShippingProcessor bsp = (BasicShippingProcessor) baseForm.getBusinessObjectFactory()
						.createObjectFromClassName(
								"com.softslate.commerce.businessobjects.shipping.BasicShippingProcessor");
				if (bsp.contingencyShippingMethodCodeDefined()) {
					Collection contingency = (Collection) bsp.loadContingencyShippingOption();
					shippingOptions.addAll(contingency);
				}

				// Use shipping rule processor to reduce the possible methods
				// available depending on shipping rules setup by the admin.
				ShippingRuleProcessor shippingRuleProcessor = (ShippingRuleProcessor) baseForm
						.getBusinessObjectFactory().createObject(
								"shippingRuleProcessorImplementer");
				int orderID = 0;
				if (baseForm.getUser() != null
						&& baseForm.getUser().getOrder() != null
						&& baseForm.getUser().getOrder().getOrderID() > 0) {
					orderID = baseForm.getUser().getOrder().getOrderID();
				}
				shippingOptions = shippingRuleProcessor
						.findAvailableShippingProcessors(shippingOptions,
								orderID);

				// Use the discount processor to apply shipping discounts to
				// each shipping option.
				CartDiscountProcessor discountProcessor = (CartDiscountProcessor) baseForm
						.getBusinessObjectFactory().createObject(
								"cartDiscountProcessorImplementer");
				shippingOptions = discountProcessor
						.processShippingDiscountsOnShippingOptions(shippingOptions);
				args.put("availableOptions", shippingOptions);

				// Now get the results
				results = shippingProcessor.processShipping(args);
			} catch (Exception ex) {
				if (log.isWarnEnabled())
					log.warn("Error calling shipping processor " + className
							+ " " + option);
				throw ex;
			}
		} else {
			results.put("resultCode", "2");
			results.put("resultMessage", "Shipping processor not found.");
		}

		// Place error messages in the request.
		String resultCode = (String) results.get("resultCode");
		String resultMessage = (String) results.get("resultMessage");
		if (resultCode == null || !resultCode.equals("0")) {
			String message = baseForm.getMessage("order",
					"shipping.processingError" + resultCode, null);
			if (message == null || message.length() == 0) {
				if (resultMessage == null) {
					message = baseForm.getMessage("order",
							"shipping.processingError", null);
				} else {
					message = baseForm.getMessage("order",
							"shipping.processingErrorArg",
							new String[] { resultMessage });
				}
			}
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", message));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): "
						+ baseForm.getErrors().toString());
		}
		return results;
	}

	public Map processTax(BaseForm baseForm) throws Exception {
		// Process tax on the cart.
		Map results = baseForm.getBusinessObjectFactory()
				.invokeMultipleProcessors("activeTaxProcessors", "processTax",
						PropertyUtils.describe(baseForm));
		return results;
	}

	public void preparePaymentForms(BaseForm baseForm, ActionServlet servlet)
			throws Exception {
		// Loop through each of the active payment forms.
		String activePaymentForms = ((Properties) baseForm.getRequest()
				.getSession().getServletContext().getAttribute("appComponents"))
				.getProperty("activePaymentForms");
		StringTokenizer st = new StringTokenizer(activePaymentForms, ",");
		int count = 0;
		while (st.hasMoreTokens()) {
			count++;
			String className = st.nextToken();
			if (log.isDebugEnabled())
				log.debug("Found payment form class: " + className);
			Class classDefinition = Class.forName(className);
			Object paymentFormObject = classDefinition.newInstance();
			ActionForm paymentForm = null;
			try {
				paymentForm = (ActionForm) paymentFormObject;
			} catch (ClassCastException cce) {
				if (log.isDebugEnabled())
					log.debug("Payment object is of the wrong type: "
							+ cce.toString());
				throw (cce);
			}
			// Populate the payment form from the request
			preparePaymentForm(paymentForm, baseForm, servlet);
			// Put the payment form in the request scope, with a unique name
			baseForm.getRequest().setAttribute("paymentForm" + count,
					paymentForm);
		}
		return;
	}

	public void preparePaymentForm(ActionForm paymentForm, BaseForm baseForm,
			ActionServlet servlet) throws Exception {
		preparePaymentForm(paymentForm, baseForm, servlet, false);
		return;
	}

	public void preparePaymentForm(ActionForm paymentForm, BaseForm baseForm,
			ActionServlet servlet, boolean validating) throws Exception {
		paymentForm.setServlet(servlet);
		RequestUtils.populate(paymentForm, baseForm.getRequest());
		Class classDefinition = paymentForm.getClass();
		Method method = MethodUtils.getAccessibleMethod(classDefinition,
				"preparePaymentFormHook", new Class[] { Boolean.class,
						BaseForm.class });
		if (method != null) {
			try {
				MethodUtils.invokeMethod(paymentForm,
						"preparePaymentFormHook", new Object[] {
						new Boolean(validating), baseForm });
			} catch (Exception ex) {
				if (log.isWarnEnabled()) {
					log.warn("Error calling preparePaymentFormHook " + classDefinition, ex);
				}
			}
		}
		return;
	}

	public ActionForward processCheckoutPayment(BaseForm baseForm,
			ActionServlet servlet) throws Exception {
		// Process the payment form that was submitted by the user.
		if (log.isDebugEnabled())
			log.debug("Finding the payment form that was submitted.");
		String activePaymentForms = ((Properties) baseForm.getRequest()
				.getSession().getServletContext().getAttribute("appComponents"))
				.getProperty("activePaymentForms");
		StringTokenizer st = new StringTokenizer(activePaymentForms, ",");
		String activePaymentProcessors = ((Properties) baseForm.getRequest()
				.getSession().getServletContext().getAttribute("appComponents"))
				.getProperty("activePaymentProcessors");
		StringTokenizer st2 = new StringTokenizer(activePaymentProcessors, ",");
		String paymentFormCount = baseForm.getRequest().getParameter(
				"paymentFormCount");
		if (paymentFormCount == null || Integer.parseInt(paymentFormCount) < 1
				|| Integer.parseInt(paymentFormCount) > st.countTokens()
				|| Integer.parseInt(paymentFormCount) > st2.countTokens()) {
			if (log.isDebugEnabled())
				log.debug("The paymentFormCount parameter was not submitted or is invalid.");
			throw (new Exception(
					"The paymentFormCount parameter was not submitted or is invalid."));
		}
		int count = 0;
		boolean foundPaymentForm = false;
		while (st.hasMoreTokens() && !foundPaymentForm) {
			count++;
			String formName = st.nextToken();
			String processorName = st2.nextToken();
			if (count == Integer.parseInt(paymentFormCount)) {
				if (log.isDebugEnabled())
					log.debug("Found payment form: " + formName);
				if (log.isDebugEnabled())
					log.debug("Found payment processor: " + processorName);
				foundPaymentForm = true;
				ActionForm paymentForm = null;
				Class classDefinition = Class.forName(formName);
				paymentForm = (ActionForm) classDefinition.newInstance();
				// Populate the form.
				preparePaymentForm(paymentForm, baseForm, servlet, true);
				// Validate the form.
				if (log.isDebugEnabled())
					log.debug("Validating the payment form.");
				if (!validatePaymentForm(paymentForm, baseForm)) {
					if (log.isDebugEnabled())
						log.debug("Payment form not valid.");
					return baseForm.getMapping().findForward(
							"failurePaymentFormValidation");
				}
				// If the store is set to use a confirm screen, set the form and
				// processor in the session and return.
				if (log.isDebugEnabled())
					log.debug("Validated the payment form.");
				String displayCheckoutConfirm = (String) baseForm.getSettings()
						.getValue("displayCheckoutConfirm");
				if (displayCheckoutConfirm.equals("1")) {
					if (log.isDebugEnabled())
						log.debug("Storing payment form and processor name in session for confirm screen.");
					baseForm.getRequest().getSession()
							.setAttribute("validatedPaymentForm", paymentForm);
					baseForm.getRequest().getSession()
							.setAttribute("processorName", processorName);
					return baseForm.getMapping().findForward(
							"successCheckoutConfirm");
				}
				// Process the payment
				if (log.isDebugEnabled())
					log.debug("Processing payment.");
				PaymentProcessor paymentProcessor = (PaymentProcessor) baseForm
						.getBusinessObjectFactory().createObjectFromClassName(
								processorName);
				int result = processPayment(paymentProcessor, paymentForm,
						baseForm);
				if (result != 0) {
					if (log.isDebugEnabled())
						log.debug("Payment processing failed.");
					// Hook for redirecting payment processors - result would be
					// 301
					if (result == 301) {
						String className = paymentProcessor.getClass()
								.getName().replaceFirst("(.*)\\.", "");
						return baseForm.getMapping().findForward(
								className + ".redirect");
					} else {
						return baseForm.getMapping().findForward(
								"failurePaymentProcess");
					}
				} else {
					if (log.isDebugEnabled())
						log.debug("Payment processed.");
					processOrderComplete(baseForm);
					return baseForm.getMapping().findForward("success");
				}
			}
		}
		processOrderComplete(baseForm);
		return baseForm.getMapping().findForward("success");
	}

	public boolean validatePaymentForm(ActionForm paymentForm, BaseForm baseForm)
			throws Exception {
		ActionErrors errors = paymentForm.validate(baseForm.getMapping(),
				baseForm.getRequest());
		if (!errors.isEmpty()) {
			saveErrors(baseForm.getRequest(), (ActionMessages) errors);
			return false;
		}
		return true;
	}

	public int processPayment(PaymentProcessor paymentProcessor,
			ActionForm paymentForm, BaseForm baseForm) throws Exception {
		// Check for a duplicate submission.
		if (baseForm.getUser() != null
				&& baseForm.getUser().getOrder() != null
				&& baseForm.getUser().getOrder().getStatus() != null
				&& (baseForm.getUser().getOrder().getStatus()
						.equals("Corrupted") || baseForm.getUser().getOrder()
						.getStatus().equals("Received"))) {
			if (log.isInfoEnabled())
				log.info("Duplicate order payment detected.");
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.duplicatePayment"));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): "
						+ baseForm.getErrors().toString());
			return 1;
		}
		// Process the payment.
		Map result = paymentProcessor.processPayment(PropertyUtils
				.describe(paymentForm));
		// Place error messages in the request.
		String resultCode = null;
		String resultMessage = null;
		Object resultCodeObj = result.get("resultCode");
		Object resultMessageObj = result.get("resultMessage");
		if (resultCodeObj != null) {
			resultCode = result.get("resultCode").toString();
		}
		if (resultMessageObj != null) {
			resultMessage = result.get("resultMessage").toString();
		}
		if ((resultCode == null || !resultCode.equals("0"))
				&& !resultCode.equals("301")) {
			String className = paymentProcessor.getClass().getName()
					.replaceFirst("(.*)\\.", "");
			String message = baseForm.getMessage("payment", "payment.error"
					+ className + resultCode, null);
			if (message == null || message.length() == 0) {
				if (resultMessage == null) {
					message = baseForm.getMessage("payment",
							"payment.processingError", null);
				} else {
					message = baseForm.getMessage("payment",
							"payment.processingErrorArg",
							new String[] { resultMessage });
				}
			}
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", message));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): "
						+ baseForm.getErrors().toString());
		}
		// In case processors return codes with letters we must leave them above
		// so the correct error messages are pulled out. But, now replace them
		// with a 9 so that it can turn into an int and we make sure it does not
		// end up as a 0 meaning successful. All that really matters here is
		// that it returns 0 or non 0.
		if (resultCode != null) {
			resultCode = resultCode.replaceAll("\\D+", "9");
		}
		return Integer.parseInt(resultCode);
	}

	public ActionForward processCheckoutConfirm(BaseForm baseForm)
			throws Exception {
		String paymentRequired = (String) baseForm.getSettings().getValue(
				"paymentRequired");
		if (paymentRequired.equals("1")
				&& baseForm.getUser().getOrder().getTotal() != null
				&& baseForm.getUser().getOrder().getTotal().doubleValue() > 0) {
			// Process the payment
			if (log.isDebugEnabled())
				log.debug("Processing payment.");
			ActionForm paymentForm = (ActionForm) baseForm.getRequest()
					.getSession().getAttribute("validatedPaymentForm");
			String processorName = (String) baseForm.getRequest().getSession()
					.getAttribute("processorName");
			if (log.isDebugEnabled())
				log.debug("processorName: " + processorName);
			PaymentProcessor paymentProcessor = (PaymentProcessor) baseForm
					.getBusinessObjectFactory().createObjectFromClassName(
							processorName);
			if (log.isDebugEnabled())
				log.debug("getBusinessObjectFactory: "
						+ baseForm.getBusinessObjectFactory());
			if (log.isDebugEnabled())
				log.debug("paymentProcessor: " + paymentProcessor);
			int result = processPayment(paymentProcessor, paymentForm, baseForm);
			// Clear the objects from the session.
			baseForm.getRequest().getSession()
					.setAttribute("validatedPaymentForm", null);
			baseForm.getRequest().getSession()
					.setAttribute("paymentProcessor", null);
			if (result != 0) {
				if (result == 301) {
					String className = paymentProcessor.getClass().getName()
							.replaceFirst("(.*)\\.", "");
					return baseForm.getMapping().findForward(
							className + ".redirect");
				} else {
					if (log.isDebugEnabled())
						log.debug("Payment processing failed.");
					return baseForm.getMapping().findForward(
							"failurePaymentProcess");
				}
			}
			if (log.isDebugEnabled())
				log.debug("Payment processed.");
		}
		processOrderComplete(baseForm);
		return baseForm.getMapping().findForward("success");
	}

	public Map processOrderComplete(BaseForm baseForm) throws Exception {

		// Process inventory and discounts
		CartProcessor cartProcessor = (CartProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"cartProcessorImplementer");
		Map invResults = cartProcessor.processOrderComplete(PropertyUtils
				.describe(baseForm));
		cartProcessor.markOrderComplete(PropertyUtils.describe(baseForm));

		// Put the user bean in the request scope to avoid another
		// request changing it.
		baseForm.getRequest().setAttribute("user", baseForm.getUser());

		// Place the baseForm in the request - used by the orderEmails.jsp file
		baseForm.getRequest().setAttribute("baseForm", baseForm);
		return invResults;
	}

	protected void saveErrors(HttpServletRequest request, ActionMessages errors) {

		// Remove any error messages attribute if none are required
		if ((errors == null) || errors.isEmpty()) {
			request.removeAttribute(Globals.ERROR_KEY);
			return;
		}

		// Save the error messages we need
		request.setAttribute(Globals.ERROR_KEY, errors);
	}

	protected void saveMessages(HttpServletRequest request,
			ActionMessages messages) {

		// Remove any messages attribute if none are required
		if ((messages == null) || messages.isEmpty()) {
			request.removeAttribute(Globals.MESSAGE_KEY);
			return;
		}

		// Save the messages we need
		request.setAttribute(Globals.MESSAGE_KEY, messages);
	}
}
