package com.softslate.commerce.customer.payment.cresecure;

import java.util.Collection;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.cresecure.CRESecureProcessor;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class for submissions of payment information for CRE Secure
 * during checkout. This form corresponds to the
 * {@link com.softslate.commerce.businessobjects.payment.cresecure.CRESecurePaymentProcessor}
 * payment processor.
 * 
 * @author Jason McWilliams
 */
public class CRESecureForm extends BaseForm {

	private static final long serialVersionUID = -2830997378657432820L;

	static Log log = LogFactory.getLog(CRESecureForm.class);

	private String ipAddress;
	private String creditCardNumberDisplay;

	public String getIpAddress() {
		return ipAddress;
	}

	public void setIpAddress(String ipAddress) {
		this.ipAddress = ipAddress;
	}

	public String getCreditCardNumberDisplay() {
		return creditCardNumberDisplay;
	}

	public void setCreditCardNumberDisplay(String creditCardNumberDisplay) {
		this.creditCardNumberDisplay = creditCardNumberDisplay;
	}

	/**
	 * Called after this form is populated. It is called before validating if it
	 * is being validated. We use this to get a token from authorize.net to show
	 * the hosted form.
	 * 
	 * @param validating
	 *            Is the form being validated or just populated and shown to the
	 *            user.
	 */
	public void preparePaymentFormHook(Boolean validating, BaseForm baseForm) throws Exception {

		// We could get the UID here if we wanted to and use that. Otherwise, we
		// will simply set the URL to get the payment form.

		if (!validating) {
			if (baseForm != null && baseForm.getRequest() != null && baseForm.getUser() != null
					&& baseForm.getUser().getOrder() != null) {
				synchronized (baseForm.getRequest().getSession()) {
					Order order = baseForm.getUser().getOrder();
					Customer customer = baseForm.getUser().getCustomer();
					CRESecureProcessor creSecureProcessor = (CRESecureProcessor) baseForm.getBusinessObjectFactory()
							.createObject("creSecureProcessorImplementer");

					// Always show embedded form in case user wants to add a new
					// card to an existing billing address. This eventually
					// should created a new billing address?

					if (order != null && StringUtils.isNotBlank(order.getPaymentProfileID())) {
						CustomerAddress creCustomerAddress = null;
						if (customer != null) {
							for (CustomerAddress addr : (Collection<CustomerAddress>) customer
									.getCustomerAddressCollection()) {
								if (StringUtils.isNotBlank(addr.getPaymentToken())
										&& order.getPaymentProfileID().equals(addr.getPaymentToken())) {
									creCustomerAddress = addr;
									break;
								}
							}
						}
						if (creCustomerAddress != null) {
							baseForm.getRequest().setAttribute("creCustomerAddress", creCustomerAddress);
							String creSecureFormURL = creSecureProcessor.prepareFormURL(order, customer, baseForm
									.getRequest().getSession().getId());
							baseForm.getRequest().setAttribute("creSecureFormURL", creSecureFormURL);
						} else {
							baseForm.getRequest().setAttribute("creSecureFormURL", null);
							baseForm.getRequest().setAttribute("creCustomerAddress", null);
							if (log.isInfoEnabled()) {
								log.info("Selected payment token is no longer found in customer address book, not allowing use.");
							}
						}
					} else {
						String creSecureFormURL = creSecureProcessor.prepareFormURL(order, customer, baseForm
								.getRequest().getSession().getId());
						baseForm.getRequest().setAttribute("creSecureFormURL", creSecureFormURL);
					}
				}
			} else {
				// We have a big problem, throw an exception
				if (log.isInfoEnabled())
					log.info("Request is null.");
				String message = getMessage("payment", "payment.errorCREMethodNotAvailable", null);
				throw new Exception(message);
			}
		}
	}


	public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		try {
			setIpAddress(getRequest().getRemoteAddr());
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());

		return getErrors();
	}


}
