package com.softslate.commerce.daos.core;

import java.io.BufferedReader;
import java.io.FileReader;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.Settings;

/**
 * 
 * @author David Tobey
 * 
 */
public class UpgradesDAOBasic extends BaseDAO {

	static Log log = LogFactory.getLog(UpgradesDAOBasic.class);

	public Settings processUpgrades(ArrayList upgrades, String pathToUpgrades,
			Settings settings) throws Exception {
		String tablePrefix = getAppSettings()
				.getProperty("databaseTablePrefix");
		// Set up the database connection and run queries within a try/catch.
		getDaoFactory().startSession();
		Statement statement1 = null;
		// Loop through each upgrade, find the file, and parse the queries
		// within them, adding each query to the Statement object's batch.
		try {
			doWorkToSetConnection(getDaoFactory().getSession());
			statement1 = getConnection().createStatement(
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_READ_ONLY);
			Iterator i = upgrades.iterator();
			SettingsDAO settingsDAO = (SettingsDAO) getDaoFactory().createDAO(
					"settingsDAOImplementer");
			settingsDAO.setConnection(getConnection());
			while (i.hasNext()) {
				Map upgrade = (Map) i.next();
				String upgradeFromVersion = (String) upgrade
						.get("upgradeFromVersion");
				String upgradeToVersion = (String) upgrade
						.get("upgradeToVersion");
				String databaseType = (String) upgrade.get("databaseType");
				if (databaseType.equals("all")) {
					String pathToScript = pathToUpgrades
							+ System.getProperty("file.separator") + "upgrade_"
							+ upgradeFromVersion + "_" + upgradeToVersion
							+ ".sql";
					if (log.isDebugEnabled())
						log.debug("Path to script: " + pathToScript);
					processFile(pathToScript, statement1, tablePrefix);
				} else {
					String pathToScript = pathToUpgrades
							+ System.getProperty("file.separator") + "upgrade_"
							+ upgradeFromVersion + "_" + upgradeToVersion + "_"
							+ databaseType + ".sql";
					if (log.isDebugEnabled())
						log.debug("Path to script: " + pathToScript);
					processFile(pathToScript, statement1, tablePrefix);
				}
				// Execute the batch of queries.
				int[] updateCounts = statement1.executeBatch();
				if (log.isDebugEnabled())
					log.debug("updateCounts: " + updateCounts);
			}
			settings = settingsDAO.loadSettings(settings);
			// Commit the transaction
			// if (log.isInfoEnabled())
			// log.info("Committing transaction...");
			// getConnection().commit();
			// if (log.isInfoEnabled())
			// log.info("Committed transaction.");
		} catch (java.sql.BatchUpdateException e) {
			// getConnection().rollback();
			log.error("Upgrades exception: ", e);
			log.error("Upgrades exception next exception: ", e.getNextException());
			throw e;
		} catch (Exception e) {
			// getConnection().rollback();
			log.error("Upgrades exception: ", e);
			log.error("Upgrades exception cause: ", e.getCause());
			throw e;
		} finally {
			try {
				if (statement1 != null)
					statement1.close();
			} catch (SQLException sqle) {
			}
		}
		return settings;
	}

	/**
	 * Add all the queries in a given file to a Statement's batch. Each query
	 * must end with a semi-colon followed immediately by a newline.
	 */
	public void processFile(String pathToScript, Statement statement,
			String tablePrefix) throws Exception {
		BufferedReader br = new BufferedReader(new FileReader(pathToScript));
		String line;
		StringBuffer contents = new StringBuffer();
		while ((line = br.readLine()) != null) {
			contents.append(line);
			if (line.trim().endsWith(";")) {
				String s = contents.toString();
				if (!tablePrefix.equals("ssc")) {
					s = s.replaceAll("ssc", tablePrefix);
				}
				// Chop off the semi-colon for Oracle
				s = s.substring(0, s.lastIndexOf(";"));
				statement.addBatch(s);
				if (log.isDebugEnabled())
					log.debug(s);
				contents = new StringBuffer();
			} else {
				contents.append("\n");
			}
		}
		br.close();
		return;
	}

	public void processSingleSqlScript(String pathToScript) throws Exception {
		String tablePrefix = getAppSettings().getProperty("databaseTablePrefix");
		getDaoFactory().startSession();
		Statement statement1 = null;
		try {
			doWorkToSetConnection(getDaoFactory().getSession());
			statement1 = getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			processFile(pathToScript, statement1, tablePrefix);
			statement1.executeBatch();
		} catch (java.sql.BatchUpdateException e) {
			log.error("Upgrades exception: ", e);
			log.error("Upgrades exception next exception: ", e.getNextException());
			throw e;
		} catch (Exception e) {
			log.error("Upgrades exception: ", e);
			log.error("Upgrades exception cause: ", e.getCause());
			throw e;
		} finally {
			try {
				if (statement1 != null) {
					statement1.close();
				}
				getDaoFactory().commitTransaction();
			} catch (SQLException sqle) {
			}
		}
	}
}
