package com.softslate.commerce.daos.order;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.order.OrderSetting;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;


/**
 * Generic DAO to manage the <code>sscOrderSetting</code> table of the
 * database.
 * 
 * @author David Tobey
 */
public class OrderSettingDAOHibernate extends BaseDAO implements
        DAOInterface, OrderSettingDAO {

    static Log log = LogFactory.getLog(OrderSettingDAOHibernate.class);

    private OrderSetting orderSetting = null;

    public OrderSetting getOrderSetting() {
        return (this.orderSetting);
    }

    public void setOrderSetting(OrderSetting orderSetting) {
        this.orderSetting = orderSetting;
    }

    public void loadOrderSettingFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            OrderSetting item = (OrderSetting) session.get(
OrderSetting.class, new Integer(getOrderSetting()
                            .getOrderSettingID()));
            PropertyUtils.copyProperties(getOrderSetting(), item);
            setOrderSetting(item);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertOrderSetting() throws DataAccessException {
        insertOrderSetting(false);
    }

    public void insertOrderSetting(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getOrderSetting());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateOrderSetting() throws DataAccessException {
        updateOrderSetting(false);
    }

    public void updateOrderSetting(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getOrderSetting());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteOrderSetting() throws DataAccessException {
        deleteOrderSetting(false);
    }

    public void deleteOrderSetting(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setOrderSetting((OrderSetting) session.load(
OrderSetting.class, new Integer(getOrderSetting()
                            .getOrderSettingID())));
            session.delete(getOrderSetting());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
