package com.softslate.commerce.daos.shipping;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to shipping methods.
 * 
 * @author David Tobey
 */
public class ShippingMethodGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate implements ShippingMethodGatewayDAO {

	static Log log = LogFactory.getLog(ShippingMethodGatewayDAOHibernate.class);

	public Map loadShippingMethodsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteShippingMethods(Map parameters, BusinessObjectFactory bof) throws Exception {
		return editDeleteItems(parameters, bof);
	}

	public Collection loadActiveMethodsAndRates() throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading active shipping methods and rates.");
		Collection methods = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "shippingMethod.selectActiveMethodsAndRates");
			methods = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return methods;
	}

	public Collection loadActiveRatesByMethodCode(String code) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading active rates by method code.");
		Collection methods = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "shippingMethod.selectActiveRatesByMethodCode");
			q.setString("code", code);
			methods = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return methods;
	}

	public Collection loadRatesByMethodCode(String code) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading rates by method code.");
		Collection methods = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "shippingMethod.selectRatesByMethodCode");
			q.setString("code", code);
			methods = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return methods;
	}
}
